@@ -23,7 +23,7 @@
 #include <stdio.h>
 #include <mysql.h>
 #include <pcre.h>
-#include <myx_aux_functions.h>
+#include <myx_util_functions.h>
 
 #include <myx_public_interface.h>
 
@@ -44,7 +44,8 @@ extern "C" {
 /// [SCRIPT::LibInterfaceMapper] -public_interface "libmysqladmin"
 #define libmysqladmin_PUBLIC_INTERFACE_VERSION 10004
 
-/// [SCRIPT::LibInterfaceMapper] -add_to_uses "myx_public_interface"
+/// [SCRIPT::LibInterfaceMapper] -add_to_uses "myx_util_public_interface, myx_public_interface"
+/// [SCRIPT::LibInterfaceMapper] -add_datatypes_from "..\..\..\mysql-gui-common\library_util\include\myx_util_public_interface.h"
 /// [SCRIPT::LibInterfaceMapper] -add_datatypes_from "..\..\..\mysql-gui-common\library\include\myx_public_interface.h"
 
 /*
@@ -101,16 +102,23 @@ typedef enum myx_backup_options
 
 typedef enum myx_backup_error
 {
-  MYX_BACKUP_NO_ERROR=0,
-  MYX_BACKUP_MYSQL_ERROR=1,
+  MYX_BACKUP_NO_ERROR,
+  MYX_BACKUP_UNKNOWN,
+  MYX_BACKUP_SERVER_ERROR,
   MYX_BACKUP_CANT_OPEN_FILE,
+  MYX_BACKUP_CANT_READ_FROM_FILE,
   MYX_BACKUP_ILLEGAL_OPTION,
   MYX_BACKUP_PCRE_ERROR,
   MYX_BACKUP_MALLOC_FAILED,
   MYX_BACKUP_OUTPUTDEVICE_FULL,
   MYX_BACKUP_CANNOT_FLUSH_TABLES_WITH_READ_LOCK,
   MYX_BACKUP_CANNOT_START_TRANSACTION,
-  MYX_BACKUP_CANNOT_SET_ANSI_QUOTES
+  MYX_BACKUP_CANNOT_SET_ANSI_QUOTES,
+  MYX_BACKUP_XML_PARSE_ERROR,
+  MYX_BACKUP_SQL_ERROR,
+  MYX_BACKUP_STOPPED,
+  MYX_BACKUP_CHARSET_CONVERSION,
+  MYX_BACKUP_WRONG_CHARSET
 } MYX_BACKUP_ERROR;
 
 typedef enum myx_expression_error
@@ -253,16 +261,16 @@ typedef struct {
 
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_VARIABLE_ELEMENT
 typedef struct {
-  unsigned char *mysql_id;
-  unsigned char *desc_id;
+  char *mysql_id;
+  char *desc_id;
   unsigned int editable;
 } MYX_VARIABLE_ELEMENT;
 
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_VARIABLES_SUBGROUP
 typedef struct {
   unsigned int pos;
-  unsigned char *name;
-  unsigned char *caption_id;
+  char *name;
+  char *caption_id;
 
   unsigned int variables_num;
   MYX_VARIABLE_ELEMENT *variables;
@@ -271,8 +279,8 @@ typedef struct {
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_VARIABLES_GROUP
 typedef struct {
   unsigned int pos;
-  unsigned char *name;
-  unsigned char *caption_id;
+  char *name;
+  char *caption_id;
 
   unsigned int variables_num;
   MYX_VARIABLE_ELEMENT *variables;
@@ -372,12 +380,18 @@ typedef struct {
 // -----------------------------------------------------------------------------------------------------------
 // Backup content
 
+#define MYX_BTF_IGNORE_CONTENT  0x00000001  // don't backup table content
+#define MYX_BTF_IS_PROCEDURE    0x00000002  // is a stored procedure
+#define MYX_BTF_IS_FUNCTION     0x00000004  // is a stored function
+#define MYX_BTF_IS_VIEW         0x00000008  // is a view
+
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_BACKUP_TABLE
 typedef struct
 {
   char *catalog;
   char *schema;
   char *table;
+  int  flags;     // collection of MYX_BTF_ constants
 } MYX_BACKUP_TABLE;
 
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_BACKUP_CONTENT
@@ -583,6 +597,8 @@ MYX_PUBLIC_FUNC int myx_free_variables_listing(MYX_VARIABLES_LISTING* variables_
 
 MYX_PUBLIC_FUNC int myx_set_variable(MYSQL *mysql, const char *mysql_id, const char *value);
 
+MYX_PUBLIC_FUNC char* myx_get_running_service_name(int port);
+
 /* cnf file functions */
 
 MYX_PUBLIC_FUNC MYX_GUI_DESCRIPTION * myx_get_gui_description(const char *filename, const char *version, MYX_PLATFORM platform, MYX_ADMIN_LIB_ERROR *error_code, const char *ini_filepath, const char *ini_file_group);
@@ -658,6 +674,8 @@ MYX_PUBLIC_FUNC int myx_save_repl_user_hosts(const MYX_USER_REPL_HOSTS *replist,
 MYX_PUBLIC_FUNC int myx_free_repl_user_hosts(MYX_USER_REPL_HOSTS *replist);
 MYX_PUBLIC_FUNC int myx_free_repl_hosts_status(MYX_REPL_HOSTS *list);
 
+MYX_PUBLIC_FUNC void myx_compact_privs(MYSQL *mysql);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
