@@ -62,6 +62,67 @@ __declspec(dllexport) BOOL WINAPI DllMain(
   }
   return TRUE;  // Successful DLL_PROCESS_ATTACH.
 }
+
+static int get_port_number_from_ini(const char *ininame)
+{
+  return (int)GetPrivateProfileInt("mysqld", "port", 0, ininame);
+}
+
+char *myx_get_running_service_name(int port)
+{
+  BOOL b;
+  SC_HANDLE srv, scm;
+  DWORD bufsz= 0, count= 0, resume= 0, i= 0;
+  ENUM_SERVICE_STATUS_PROCESS *next;
+  char *pbuf, *pname, *ppath, *ppathend, *ininame;
+  char delim;
+  LPQUERY_SERVICE_CONFIG pconf;
+  
+  scm= OpenSCManager(NULL, NULL, GENERIC_READ);
+  EnumServicesStatusEx(scm, SC_ENUM_PROCESS_INFO, SERVICE_WIN32, SERVICE_ACTIVE, NULL, 0, 
+    &bufsz, &count, &resume, NULL);
+  pbuf= g_malloc(bufsz);
+  b= EnumServicesStatusEx(scm, SC_ENUM_PROCESS_INFO, SERVICE_WIN32, SERVICE_ACTIVE, pbuf, bufsz, 
+    &bufsz, &count, &resume, NULL);
+
+  for(i= 0; i < count; i++)
+  {
+    next= ((ENUM_SERVICE_STATUS_PROCESS *)pbuf) + i;
+    pname= _strlwr(g_strdup(next->lpServiceName));
+    // find a running "*mysql*" service
+    if((strstr(pname, "mysql") != NULL) && (next->ServiceStatusProcess.dwCurrentState & SERVICE_RUNNING)) 
+    {
+      srv= OpenService(scm, next->lpServiceName, GENERIC_READ);
+      QueryServiceConfig(srv, NULL, 0, &bufsz);
+      pconf= g_malloc(bufsz);
+      b= QueryServiceConfig(srv, pconf, bufsz, &bufsz);
+      ppath= strstr(pconf->lpBinaryPathName, "--defaults-file=") + sizeof("--defaults-file=") - 1;
+      delim= *ppath++;
+      ppathend= ppath;
+      while(*ppathend && *ppathend != delim) 
+      {
+        ppathend++;
+      }
+      ininame= memcpy(g_malloc((gulong)(ppathend - ppath + 1)), ppath, ppathend - ppath);
+      ininame[ppathend - ppath]= '\0';
+      if(get_port_number_from_ini(ininame) == port)
+      {
+        g_free(ininame);
+        g_free(pconf);
+        CloseServiceHandle(srv);
+        CloseServiceHandle(scm);
+        return pname;
+      }
+      CloseServiceHandle(srv);
+      g_free(ininame);
+      g_free(pconf);
+    }
+    g_free(pname);
+  }
+
+  CloseServiceHandle(scm);
+  return g_strdup("");
+}
 #endif
 
 
@@ -270,7 +331,6 @@ int myx_set_variable(MYSQL *mysql, const char *mysql_id, const char *value)
   return res;
 }
 
-
 static MYX_TABLE_COMMAND_STATUSES *do_table_command(MYSQL *mysql, const char *command)
 {
   MYX_TABLE_COMMAND_STATUSES *status= NULL;
@@ -288,8 +348,8 @@ static MYX_TABLE_COMMAND_STATUSES *do_table_command(MYSQL *mysql, const char *co
 
     while ((row= mysql_fetch_row(res)))
     {
-      status->status[i].table= DBSTR(mysql, row[0]);
-      status->status[i].message= DBSTR(mysql, row[3]);
+      status->status[i].table= myx_convert_dbstr_utf8(mysql, row[0]);
+      status->status[i].message= myx_convert_dbstr_utf8(mysql, row[3]);
 
       if (strcmp2(row[2],"status")==0)
         status->status[i].message_type= MYX_MESSAGE_STATUS;
