@@ -101,8 +101,8 @@ void MABackupPanel::setup_selection_list()
                                               get_app_file("option_checked.xpm"));
   
   // schedule buttons
-  get_button("install_button")->signal_clicked().connect(SigC::slot(*this,&MABackupPanel::install_schedule));
-  get_button("uninstall_button")->signal_clicked().connect(SigC::slot(*this,&MABackupPanel::uninstall_schedule));
+  get_button("install_button")->signal_clicked().connect(sigc::mem_fun(*this,&MABackupPanel::install_schedule));
+  get_button("uninstall_button")->signal_clicked().connect(sigc::mem_fun(*this,&MABackupPanel::uninstall_schedule));
   
   // setup list columns
 
@@ -115,7 +115,7 @@ void MABackupPanel::setup_selection_list()
   column= new Gtk::TreeView::Column(_("Objects"), *Gtk::manage(trend));
   tree->append_column(*Gtk::manage(column));
   column->add_attribute(trend->property_state(), _sel_columns._state);
-  trend->signal_toggled().connect(SigC::slot(*this,&MABackupPanel::row_toggled));
+  trend->signal_toggled().connect(sigc::mem_fun(*this,&MABackupPanel::row_toggled));
   
   column= tree->get_column(0);
   column->pack_start(_sel_columns._icon, false);
@@ -132,7 +132,7 @@ void MABackupPanel::setup_selection_list()
 
   Glib::RefPtr<Gtk::TreeSelection> sel= tree->get_selection();
 
-  sel->signal_changed().connect(SigC::slot(*this,&MABackupPanel::selected_object));
+  sel->signal_changed().connect(sigc::mem_fun(*this,&MABackupPanel::selected_object));
   
 }
 
@@ -147,7 +147,7 @@ bool MABackupPanel::init()
   if (!MAPanel::init_from_glade(get_glade_file(GLADE_BACKUP_FILE), "panel_frame"))
     return false;
 
-  _progress_xml= new MGGladeXML(get_glade_file(GLADE_BACKUP_FILE), "progress_dialog");
+  _progress_xml= new MGGladeXML(get_glade_file(GLADE_BACKUP_FILE), "progress_dialog", "mysql-administrator");
   if (!_progress_xml)
   {
     g_error(_("Could not load backup progress dialog"));
@@ -161,7 +161,7 @@ bool MABackupPanel::init()
   _file_browser= new MGFileBrowserList(_("Backup Projects"),
                                        profile_dir);
   _file_browser->set_extension(PROFILE_SUFFIX);
-  _file_browser->signal_selected().connect(SigC::slot(*this,&MABackupPanel::profile_selected));
+  _file_browser->signal_selected().connect(sigc::mem_fun(*this,&MABackupPanel::profile_selected));
 
   _file_browser->set_empty_message(_("Click 'New Project' to create a Backup Profile."));
   
@@ -172,21 +172,21 @@ bool MABackupPanel::init()
 
     image= Gtk::manage(new Gtk::Image(Gtk::Stock::NEW, Gtk::ICON_SIZE_MENU));
     item= Gtk::manage(new Gtk::ImageMenuItem(*image, _("_New Profile"), true));
-    item->signal_activate().connect(SigC::slot(*this,&MABackupPanel::create_profile));
+    item->signal_activate().connect(sigc::mem_fun(*this,&MABackupPanel::create_profile));
     image->show();
     item->show();
     menu->append(*item);
 
     image= Gtk::manage(new Gtk::Image(Gtk::Stock::COPY, Gtk::ICON_SIZE_MENU));
     item= Gtk::manage(new Gtk::ImageMenuItem(*image, _("_Clone Profile"), true));
-    item->signal_activate().connect(SigC::slot(*this,&MABackupPanel::clone_profile));
+    item->signal_activate().connect(sigc::mem_fun(*this,&MABackupPanel::clone_profile));
     image->show();
     item->show();
     menu->append(*item);
 
     image= Gtk::manage(new Gtk::Image(Gtk::Stock::DELETE, Gtk::ICON_SIZE_MENU));
     item= Gtk::manage(new Gtk::ImageMenuItem(*image, _("_Delete Profile"), true));
-    item->signal_activate().connect(SigC::slot(*this,&MABackupPanel::delete_profile));
+    item->signal_activate().connect(sigc::mem_fun(*this,&MABackupPanel::delete_profile));
     image->show();
     item->show();
     menu->append(*item);
@@ -200,6 +200,8 @@ bool MABackupPanel::init()
   _sys_schema_icon= _schema_icon;
 
   _table_icon= PIXCACHE->load("16x16_Table.png");
+  _view_icon= PIXCACHE->load("16x16_View.png");
+  _proc_icon= PIXCACHE->load("16x16_StoredProc.png");
   _column_icon= PIXCACHE->load("16x16_Field.png");
 
   _schema_browser= new MGTableBrowserList("", MGTableBrowserList::Schema);
@@ -207,57 +209,57 @@ bool MABackupPanel::init()
   _schema_browser->set_name("schema_list");
   
   ((Gtk::Container*)get_widget("schemata_frame"))->add(*Gtk::manage(_schema_browser));
-  _schema_browser->signal_selected().connect(SigC::slot(*this,&MABackupPanel::schema_selected));
+  _schema_browser->signal_selected().connect(sigc::mem_fun(*this,&MABackupPanel::schema_selected));
  // _schema_browser->set_icons(_schema_icon, _sys_schema_icon, _table_icon, _column_icon);
   _schema_browser->show();
 
-  _data->signal_catalogs_refreshed().connect(SigC::slot(*this,&MABackupPanel::schemata_reloaded));
+  _data->signal_catalogs_refreshed().connect(sigc::mem_fun(*this,&MABackupPanel::schemata_reloaded));
 
-  get_button("target_button")->signal_clicked().connect(SigC::slot(*this,&MABackupPanel::select_target_path));
-  get_toggle("backup_all_check")->signal_toggled().connect(SigC::slot(*this,&MABackupPanel::backup_all_check_toggled));
+  get_button("target_button")->signal_clicked().connect(sigc::mem_fun(*this,&MABackupPanel::select_target_path));
+  get_toggle("backup_all_check")->signal_toggled().connect(sigc::mem_fun(*this,&MABackupPanel::backup_all_check_toggled));
 
   // setup signals
   Gtk::Button *btn;
   
   btn= (Gtk::Button*)_progress_xml->get_widget("stop_button");
-  btn->signal_clicked().connect(SigC::slot(*this,&MABackupPanel::stop_backup));
+  btn->signal_clicked().connect(sigc::mem_fun(*this,&MABackupPanel::stop_backup));
   
   btn= get_button("add_button");
-  btn->signal_clicked().connect(SigC::slot(*this,&MABackupPanel::add_object));
+  btn->signal_clicked().connect(sigc::mem_fun(*this,&MABackupPanel::add_object));
 
   btn= get_button("remove_button");
-  btn->signal_clicked().connect(SigC::slot(*this,&MABackupPanel::remove_object));
+  btn->signal_clicked().connect(sigc::mem_fun(*this,&MABackupPanel::remove_object));
 
   btn= get_button("new_button");
-  btn->signal_clicked().connect(SigC::slot(*this,&MABackupPanel::create_profile));
+  btn->signal_clicked().connect(sigc::mem_fun(*this,&MABackupPanel::create_profile));
 
   btn= get_button("import_button");
-  btn->signal_clicked().connect(SigC::slot(*this,&MABackupPanel::import_profile));
+  btn->signal_clicked().connect(sigc::mem_fun(*this,&MABackupPanel::import_profile));
 
   btn= get_button("save_button");
-  btn->signal_clicked().connect(SigC::slot(*this,&MABackupPanel::save_profile));
+  btn->signal_clicked().connect(sigc::mem_fun(*this,&MABackupPanel::save_profile));
 
   btn= get_button("start_button");
-  btn->signal_clicked().connect(SigC::slot(*this,&MABackupPanel::perform_backup));
+  btn->signal_clicked().connect(sigc::mem_fun(*this,&MABackupPanel::perform_backup));
 
-  get_entry("name_entry")->signal_changed().connect(SigC::slot(*this,&MABackupPanel::set_dirty));
-  get_entry("name_entry")->signal_changed().connect(SigC::slot(*this,&MABackupPanel::name_changed));
+  get_entry("name_entry")->signal_changed().connect(sigc::mem_fun(*this,&MABackupPanel::set_dirty));
+  get_entry("name_entry")->signal_changed().connect(sigc::mem_fun(*this,&MABackupPanel::name_changed));
   
   for (int i= 1; i <= 8; i++)
   {
     char buf[32];
     sprintf(buf, "option%i_check", i);
-    ((Gtk::CheckButton*)get_widget(buf))->signal_toggled().connect(SigC::slot(*this,&MABackupPanel::set_dirty));
+    ((Gtk::CheckButton*)get_widget(buf))->signal_toggled().connect(sigc::mem_fun(*this,&MABackupPanel::set_dirty));
   }
 
-  ((Gtk::RadioButton*)get_widget("lockall_radio"))->signal_toggled().connect(SigC::slot(*this,&MABackupPanel::set_dirty));
-  ((Gtk::RadioButton*)get_widget("singletrans_radio"))->signal_toggled().connect(SigC::slot(*this,&MABackupPanel::set_dirty));
-  ((Gtk::RadioButton*)get_widget("normal_radio"))->signal_toggled().connect(SigC::slot(*this,&MABackupPanel::set_dirty));
-  ((Gtk::RadioButton*)get_widget("binlogpos_radio"))->signal_toggled().connect(SigC::slot(*this,&MABackupPanel::set_dirty));
+  ((Gtk::RadioButton*)get_widget("lockall_radio"))->signal_toggled().connect(sigc::mem_fun(*this,&MABackupPanel::set_dirty));
+  ((Gtk::RadioButton*)get_widget("singletrans_radio"))->signal_toggled().connect(sigc::mem_fun(*this,&MABackupPanel::set_dirty));
+  ((Gtk::RadioButton*)get_widget("normal_radio"))->signal_toggled().connect(sigc::mem_fun(*this,&MABackupPanel::set_dirty));
+  ((Gtk::RadioButton*)get_widget("binlogpos_radio"))->signal_toggled().connect(sigc::mem_fun(*this,&MABackupPanel::set_dirty));
 
-  get_toggle("backup_all_check")->signal_toggled().connect(SigC::slot(*this,&MABackupPanel::set_dirty));
+  get_toggle("backup_all_check")->signal_toggled().connect(sigc::mem_fun(*this,&MABackupPanel::set_dirty));
   
-  ((Gtk::OptionMenu*)get_widget("schedule_menu"))->signal_changed().connect(SigC::slot(*this,&MABackupPanel::schedule_type_changed));
+  ((Gtk::OptionMenu*)get_widget("schedule_menu"))->signal_changed().connect(sigc::mem_fun(*this,&MABackupPanel::schedule_type_changed));
   schedule_type_changed();
   
   update_sensitivity(false);
@@ -480,36 +482,77 @@ void MABackupPanel::add_schema_to_list(const Glib::ustring &catalog,
   s_row[_sel_columns._what]= SelectionColumns::CSchema;
 
   // fetch table list for schema
-  MYX_SCHEMA_TABLE_STATUS *tables= _data->get_schema_table_status(catalog,schema);
+  MYX_SCHEMA_ENTITY_STATUS *entities= _data->get_schema_entity_status(catalog,schema);
 
   // add tables of this schema
-  for (unsigned int i= 0; tables && i < tables->schema_tables_num; ++i)
+  for (unsigned int i= 0; entities && i < entities->schema_entities_num; ++i)
   {
-    MYX_TABLE_STATUS *table= &tables->schema_tables[i];
-
-    t_iter= _selection_store->append(s_row.children());
-    t_row= *t_iter;
-
-    t_row[_sel_columns._icon]= _table_icon;
-    t_row[_sel_columns._object]= (char*)table->table_name;
-    t_row[_sel_columns._type]= (table->table_type)? ((char*)table->table_type):"";
-    t_row[_sel_columns._rows]= atoll((char*)table->rows);
-    t_row[_sel_columns._length]= format_value(atoll((char*)table->data_length));
-    t_row[_sel_columns._time]= (char*)table->update_time;
-
-    t_row[_sel_columns._lengthn]= atoll((char*)table->data_length);
-    t_row[_sel_columns._state]= checked?MGCellRendererTristate::ON:MGCellRendererTristate::OFF;
-    t_row[_sel_columns._what]= SelectionColumns::CTable;
-    
-    total_rows+= atoll((char*)table->rows);
-    total_length+= atoll((char*)table->data_length);
+    if(entities->schema_entities[i].entity_type == MYX_ENTITY_TABLE)
+    {
+      MYX_TABLE_STATUS *table= (MYX_TABLE_STATUS *)entities->schema_entities[i].entity;
+  
+      t_iter= _selection_store->append(s_row.children());
+      t_row= *t_iter;
+  
+      t_row[_sel_columns._icon]= _table_icon;
+      t_row[_sel_columns._object]= (char*)table->table_name;
+      t_row[_sel_columns._type]= (table->table_type)? ((char*)table->table_type):"";
+      t_row[_sel_columns._rows]= atoll((char*)table->rows);
+      t_row[_sel_columns._length]= format_value(atoll((char*)table->data_length));
+      t_row[_sel_columns._time]= (char*)table->update_time;
+  
+      t_row[_sel_columns._lengthn]= atoll((char*)table->data_length);
+      t_row[_sel_columns._state]= checked?MGCellRendererTristate::ON:MGCellRendererTristate::OFF;
+      t_row[_sel_columns._what]= SelectionColumns::CTable;
+      
+      total_rows+= atoll((char*)table->rows);
+      total_length+= atoll((char*)table->data_length);
+    }
+    else if(entities->schema_entities[i].entity_type == MYX_ENTITY_VIEW)
+    {
+      MYX_VIEW_STATUS *view= (MYX_VIEW_STATUS *)entities->schema_entities[i].entity;
+  
+      t_iter= _selection_store->append(s_row.children());
+      t_row= *t_iter;
+  
+      t_row[_sel_columns._icon]= _view_icon;
+      t_row[_sel_columns._object]= (char*)view->view_name;
+      t_row[_sel_columns._type]= "VIEW";
+      t_row[_sel_columns._rows]= 0;
+      t_row[_sel_columns._length]= "";
+      t_row[_sel_columns._time]= "";
+      t_row[_sel_columns._lengthn]= 0;
+      t_row[_sel_columns._state]= checked?MGCellRendererTristate::ON:MGCellRendererTristate::OFF;
+      t_row[_sel_columns._what]= SelectionColumns::CView;
+    }
+    else if((entities->schema_entities[i].entity_type == MYX_ENTITY_PROC) ||
+      (entities->schema_entities[i].entity_type == MYX_ENTITY_FUNC))
+    {
+      MYX_SCHEMA_STORED_PROCEDURE *sp= 
+      (MYX_SCHEMA_STORED_PROCEDURE *)entities->schema_entities[i].entity;
+  
+      t_iter= _selection_store->append(s_row.children());
+      t_row= *t_iter;
+  
+      t_row[_sel_columns._icon]= _proc_icon;
+      t_row[_sel_columns._object]= (char*)sp->name;
+      t_row[_sel_columns._type]= 
+        entities->schema_entities[i].entity_type == MYX_ENTITY_PROC 
+        ? "PROCEDURE":"FUNCTION";
+      t_row[_sel_columns._rows]= 0;
+      t_row[_sel_columns._length]= "";
+      t_row[_sel_columns._time]= sp->modified;
+      t_row[_sel_columns._lengthn]= 0;
+      t_row[_sel_columns._state]= checked?MGCellRendererTristate::ON:MGCellRendererTristate::OFF;
+      t_row[_sel_columns._what]= SelectionColumns::CRoutine;
+    }
   }
 
   s_row[_sel_columns._rows]= total_rows;
   s_row[_sel_columns._lengthn]= total_length;
   s_row[_sel_columns._length]= format_value(total_length);
   
-  myx_free_schema_table_status(tables);
+  myx_free_schema_entity_status(entities);
 }
 
 
@@ -795,7 +838,7 @@ void MABackupPanel::show_profile(MYX_BACKUP_PROFILE *profile)
 MYX_BACKUP_CONTENT *MABackupPanel::gather_selected_tables()
 {
   Gtk::TreeIter siter, titer;
-  int i;
+  unsigned int i;
   MYX_BACKUP_CONTENT *content;
   GPtrArray *ta= g_ptr_array_new();
   Glib::ustring str;
@@ -815,6 +858,7 @@ MYX_BACKUP_CONTENT *MABackupPanel::gather_selected_tables()
       bt->catalog= g_strdup("def");//XXX
       bt->schema= g_strdup(str.c_str());
       bt->table= NULL;
+      bt->flags= 0;
       g_ptr_array_add(ta, bt);
     }
     else
@@ -831,7 +875,25 @@ MYX_BACKUP_CONTENT *MABackupPanel::gather_selected_tables()
           str= row[_sel_columns._object];
           bt->schema= g_strdup(str.c_str());
           str= rrow[_sel_columns._object];
-          bt->table= g_strdup(str.c_str());;
+          bt->table= g_strdup(str.c_str());
+
+          str= rrow[_sel_columns._type];
+          if(strcmp(str.c_str(), "VIEW") == 0) 
+          {
+            bt->flags= MYX_BTF_IS_VIEW;
+          }
+          else if(strcmp(str.c_str(), "PROCEDURE") == 0) 
+          {
+            bt->flags= MYX_BTF_IS_PROCEDURE;
+          }
+          else if(strcmp(str.c_str(), "FUNCTION") == 0)
+          {
+            bt->flags= MYX_BTF_IS_FUNCTION;
+          }
+          else
+          {
+            bt->flags= 0;
+          }
           g_ptr_array_add(ta, bt);
         }
       }
@@ -849,6 +911,7 @@ MYX_BACKUP_CONTENT *MABackupPanel::gather_selected_tables()
     content->tables[i].catalog= bt->catalog;
     content->tables[i].schema= bt->schema;
     content->tables[i].table= bt->table;
+    content->tables[i].flags= bt->flags;
   }
 
   return content;
@@ -907,6 +970,7 @@ void MABackupPanel::save_profile()
   {
     Gtk::MessageDialog dlg(*_app->window(),
                            _("Please supply a name for this profile"),
+                           false,
                            Gtk::MESSAGE_ERROR,
                            Gtk::BUTTONS_OK,
                            true);
@@ -916,6 +980,7 @@ void MABackupPanel::save_profile()
   {
     Gtk::MessageDialog dlg(*_app->window(),
                            _("Profile name cannot contain character '/'"),
+                           false,
                            Gtk::MESSAGE_ERROR,
                            Gtk::BUTTONS_OK,
                            true);
@@ -1201,8 +1266,8 @@ void MABackupPanel::perform_backup()
 
   if (_inst->check_connection())
   {
-    SigC::Connection con= Glib::signal_timeout().connect(SigC::bind<void*>
-                                                         (SigC::slot(*this, &MABackupPanel::update_backup_progress),&arg.pdata),
+    SigC::Connection con= Glib::signal_timeout().connect(sigc::bind<void*>
+                                                         (sigc::mem_fun(*this, &MABackupPanel::update_backup_progress),&arg.pdata),
                                                          50);
     dlg->show();
 
@@ -1213,7 +1278,7 @@ void MABackupPanel::perform_backup()
     con.disconnect();
   }
   else
-    err= MYX_BACKUP_MYSQL_ERROR;
+    err= MYX_BACKUP_SERVER_ERROR;
 
   myx_free_profile(arg.profile);
   
@@ -1231,6 +1296,7 @@ void MABackupPanel::perform_backup()
     Gtk::MessageDialog dlg(*_app->window(),
                            ufmt(_("Backup written to file '%s'."),
                                 arg.path.c_str()),
+                           false,
                            Gtk::MESSAGE_INFO,
                            Gtk::BUTTONS_OK,
                            true);
@@ -1238,7 +1304,8 @@ void MABackupPanel::perform_backup()
   } 
   else
   {
-    Gtk::MessageDialog dlg(*_app->window(), _("Error during backup.\n")+msg,
+    Gtk::MessageDialog dlg(*_app->window(), _("<b>Error during Backup</b>\n")+msg,
+                           true,
                            Gtk::MESSAGE_ERROR,
                            Gtk::BUTTONS_OK,
                            true);
@@ -1259,12 +1326,12 @@ void MABackupPanel::schedule_type_changed()
 bool MABackupPanel::gather_schedule(MCrontab::Entry &schedule)
 {
   extern std::string get_argv0(); // from main.cc
-  
+  extern std::string get_startup_dir(); // from main.cc
+
   switch (static_cast<Gtk::OptionMenu*>(get_widget("schedule_menu"))->get_history())
   {
   case 0:
     return false;
-    
   case 1:
     schedule.hour= get_spin("dailyh_spin")->get_value_as_int();
     schedule.minute= get_spin("dailym_spin")->get_value_as_int();
@@ -1302,7 +1369,7 @@ bool MABackupPanel::gather_schedule(MCrontab::Entry &schedule)
   if (Glib::str_has_suffix(schedule.command, "mysql-administrator-bin"))
     schedule.command= schedule.command.substr(0, schedule.command.size()-4);
 #else
-  schedule.command= get_argv0();
+schedule.command= get_argv0();
   if (Glib::str_has_suffix(schedule.command, "mysql-administrator"))
   {
     schedule.command= schedule.command.substr(0, 
@@ -1318,6 +1385,11 @@ bool MABackupPanel::gather_schedule(MCrontab::Entry &schedule)
     schedule.command= "/usr/local/";
   }
   schedule.command+= "mabackup";
+
+  if(get_argv0()[0] != '/')
+  {
+    schedule.command = get_startup_dir() + "/" + schedule.command;
+  }
 #endif
 
   if (!target.empty())
@@ -1600,7 +1672,7 @@ Glib::ustring MABackupPanel::get_backup_error_text(MYSQL *mysql, MYX_BACKUP_ERRO
   
   switch (err)
   {
-  case MYX_BACKUP_MYSQL_ERROR:
+  case MYX_BACKUP_SERVER_ERROR:
     //msg= _("MySQL error during backup.");
     msg= myx_mysql_error(mysql);
     break;
@@ -1628,6 +1700,27 @@ Glib::ustring MABackupPanel::get_backup_error_text(MYSQL *mysql, MYX_BACKUP_ERRO
   case MYX_BACKUP_CANNOT_SET_ANSI_QUOTES:
     msg= _("Cannot set ANSI quotes.");
     break;
+  case MYX_BACKUP_CANT_READ_FROM_FILE:
+    msg= _("Cannot read from file.");
+    break;
+  case MYX_BACKUP_XML_PARSE_ERROR:
+    msg= _("Error parsing XML file.");
+    break;
+  case MYX_BACKUP_SQL_ERROR:
+    msg= _("SQL error");
+    break;
+  case MYX_BACKUP_STOPPED:
+    msg= _("Stopped by user.");
+    break;
+  case MYX_BACKUP_CHARSET_CONVERSION:
+    msg= _("Character set conversion error.");
+    break;
+  case MYX_BACKUP_WRONG_CHARSET:
+    msg= _("Wrong character set.");
+    break;
+  case MYX_BACKUP_UNKNOWN:
+    msg= _("Unknown error.");
+    break;
   case MYX_BACKUP_NO_ERROR:
     break;
   }
@@ -1722,3 +1815,4 @@ MAPanel *create_backup_panel(MAdministrator *app, MDataInterface *data)
 {
   return new MABackupPanel(app, data);
 }
+
