@@ -38,7 +38,9 @@ class MABackupPanel : public MAPanel {
       public:
         enum What {
           CSchema,
-          CTable
+          CTable,
+          CView,
+          CRoutine
         };
 
         SelectionColumns() {
@@ -62,6 +64,8 @@ class MABackupPanel : public MAPanel {
     Glib::RefPtr<Gdk::Pixbuf> _schema_icon;
     Glib::RefPtr<Gdk::Pixbuf> _sys_schema_icon;
     Glib::RefPtr<Gdk::Pixbuf> _table_icon;
+    Glib::RefPtr<Gdk::Pixbuf> _view_icon;
+    Glib::RefPtr<Gdk::Pixbuf> _proc_icon;
     Glib::RefPtr<Gdk::Pixbuf> _column_icon;
 
     MGTableBrowserList *_schema_browser;
