@@ -302,11 +302,13 @@ class BarGraph : public MAHealthPanel::BaseGraph {
         _graph->set_current_format("");
 
       _graph->set_value_unit((MGHMeterGraphPlotter::Unit)graph->value_unit);
+      // this caused behaviour described in bug #11631 (maybe not a bug?)
+      /*
       if (graph->max_caption)
         _graph->set_total_format(Glib::ustring((char*)graph->max_caption));
       else
         _graph->set_total_format("");
-
+      */
       _expr= graph->value_formula?:"";
       _max_expr= graph->max_formula?:"";
 
@@ -391,58 +393,60 @@ MAHealthPanel::MAHealthPanel(MAdministrator *app, MDataInterface *data)
 
   // setup graph menu
   item= Gtk::manage(new Gtk::MenuItem(_("Add Graph")));
-  item->signal_activate().connect(SigC::slot(*this,&MAHealthPanel::add_graph_mi));
+  item->signal_activate().connect(sigc::mem_fun(*this,&MAHealthPanel::add_graph_mi));
   _graph_menu.append(*item);
-//  item= Gtk::manage(new Gtk::MenuItem(_("Edit Graph")));
-//  item->signal_activate().connect(SigC::slot(*this,&MAHealthPanel::edit_graph_mi));
-//  _graph_menu.append(*item);
+
+  item= Gtk::manage(new Gtk::MenuItem(_("Edit Graph")));
+  item->signal_activate().connect(sigc::mem_fun(*this,&MAHealthPanel::edit_graph_mi));
+  _graph_menu.append(*item);
+
   _graph_menu.append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
   item= Gtk::manage(new Gtk::ImageMenuItem(*Gtk::manage(new Gtk::Image(Gtk::Stock::DELETE,Gtk::ICON_SIZE_MENU)),
                                            _("Delete Graph")));
-  item->signal_activate().connect(SigC::slot(*this,&MAHealthPanel::delete_graph_mi));
+  item->signal_activate().connect(sigc::mem_fun(*this,&MAHealthPanel::delete_graph_mi));
   _graph_menu.append(*item);
   _graph_menu.show_all();
-  
+
   // setup group menu
   item= Gtk::manage(new Gtk::MenuItem(_("Add Group")));
-  item->signal_activate().connect(SigC::slot(*this,&MAHealthPanel::add_group_mi));
+  item->signal_activate().connect(sigc::mem_fun(*this,&MAHealthPanel::add_group_mi));
   _group_menu.append(*item);
   item= Gtk::manage(new Gtk::MenuItem(_("Rename Group")));
-  item->signal_activate().connect(SigC::slot(*this,&MAHealthPanel::rename_group_mi));
+  item->signal_activate().connect(sigc::mem_fun(*this,&MAHealthPanel::rename_group_mi));
   _group_menu.append(*item);
   item= Gtk::manage(new Gtk::MenuItem(_("Add Graph")));
-  item->signal_activate().connect(SigC::slot(*this,&MAHealthPanel::add_graph_mi));
+  item->signal_activate().connect(sigc::mem_fun(*this,&MAHealthPanel::add_graph_mi));
   _group_menu.append(*item);
   _group_menu.append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
   item= Gtk::manage(new Gtk::ImageMenuItem(*Gtk::manage(new Gtk::Image(Gtk::Stock::DELETE,Gtk::ICON_SIZE_MENU)),
                                            _("Delete Group")));
-  item->signal_activate().connect(SigC::slot(*this,&MAHealthPanel::delete_group_mi));
+  item->signal_activate().connect(sigc::mem_fun(*this,&MAHealthPanel::delete_group_mi));
   _group_menu.append(*item);
   _group_menu.show_all();
   
   // setup page menu
   item= Gtk::manage(new Gtk::MenuItem(_("Add Page")));
-  item->signal_activate().connect(SigC::slot(*this,&MAHealthPanel::add_page_mi));
+  item->signal_activate().connect(sigc::mem_fun(*this,&MAHealthPanel::add_page_mi));
   _page_menu.append(*item);
   item= Gtk::manage(new Gtk::MenuItem(_("Edit Page")));
-  item->signal_activate().connect(SigC::slot(*this,&MAHealthPanel::rename_page_mi));
+  item->signal_activate().connect(sigc::mem_fun(*this,&MAHealthPanel::rename_page_mi));
   _page_menu.append(*item);
   item= Gtk::manage(new Gtk::MenuItem(_("Add Group")));
-  item->signal_activate().connect(SigC::slot(*this,&MAHealthPanel::add_group_mi));
+  item->signal_activate().connect(sigc::mem_fun(*this,&MAHealthPanel::add_group_mi));
   _page_menu.append(*item);
   _page_menu.append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
   item= Gtk::manage(new Gtk::ImageMenuItem(*Gtk::manage(new Gtk::Image(Gtk::Stock::DELETE,Gtk::ICON_SIZE_MENU)),
                                            _("Delete Page")));
-  item->signal_activate().connect(SigC::slot(*this,&MAHealthPanel::delete_page_mi));
+  item->signal_activate().connect(sigc::mem_fun(*this,&MAHealthPanel::delete_page_mi));
   _page_menu.append(*item);
   _page_menu.append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
   item= Gtk::manage(new Gtk::ImageMenuItem(*Gtk::manage(new Gtk::Image(Gtk::Stock::UNDO,Gtk::ICON_SIZE_MENU)),
                                            _("Restore Defaults")));
-  item->signal_activate().connect(SigC::slot(*this,&MAHealthPanel::restore_defaults_mi));
+  item->signal_activate().connect(sigc::mem_fun(*this,&MAHealthPanel::restore_defaults_mi));
   _page_menu.append(*item);
   _page_menu.show_all();
   
-  _instance->signal_disconnect().connect(SigC::slot(*this,&MAHealthPanel::handle_disconnect));
+  _instance->signal_disconnect().connect(sigc::mem_fun(*this,&MAHealthPanel::handle_disconnect));
 }
 
 
@@ -593,8 +597,8 @@ bool MAHealthPanel::init()
   setup_type_tree(VServer);
 
   // setup other signals
-  get_button("refresh_status_button")->signal_clicked().connect(SigC::bind<int>(SigC::slot(*this,&MAHealthPanel::refresh_list),0));
-  get_button("refresh_server_button")->signal_clicked().connect(SigC::bind<int>(SigC::slot(*this,&MAHealthPanel::refresh_list),1));
+  get_button("refresh_status_button")->signal_clicked().connect(sigc::bind<int>(sigc::mem_fun(*this,&MAHealthPanel::refresh_list),0));
+  get_button("refresh_server_button")->signal_clicked().connect(sigc::bind<int>(sigc::mem_fun(*this,&MAHealthPanel::refresh_list),1));
   
   return true;
 }
@@ -682,8 +686,8 @@ void MAHealthPanel::setup_type_tree(int index)
   if (store->children().begin())
     sel->select(store->children().begin());
   sel->set_mode(Gtk::SELECTION_BROWSE);
-  sel->signal_changed().connect(SigC::bind<int>
-                               (SigC::slot(*this, &MAHealthPanel::type_selected),
+  sel->signal_changed().connect(sigc::bind<int>
+                               (sigc::mem_fun(*this, &MAHealthPanel::type_selected),
                                 index));
 }
 
@@ -702,7 +706,7 @@ void MAHealthPanel::setup_value_list(Gtk::TreeView *tree)
   rend= (Gtk::CellRendererText*)*column->get_cell_renderers().begin();
   column->add_attribute(rend->property_editable(),
                         _value_columns._editable);
-  rend->signal_edited().connect(SigC::slot(*this,&MAHealthPanel::variable_edited));
+  rend->signal_edited().connect(sigc::mem_fun(*this,&MAHealthPanel::variable_edited));
 
   tree->append_column(_("Description"), _value_columns._descr);
 
@@ -750,7 +754,7 @@ void MAHealthPanel::add_page(MYX_HEALTH_PAGE *page)
 
   Gtk::EventBox *evbox= Gtk::manage(new Gtk::EventBox);
   evbox->add(*page_info->box);
-  evbox->signal_button_press_event().connect(SigC::bind<GraphPage*>(SigC::slot(*this,&MAHealthPanel::page_button_press),
+  evbox->signal_button_press_event().connect(sigc::bind<GraphPage*>(sigc::mem_fun(*this,&MAHealthPanel::page_button_press),
                                                                    page_info));
 
   evbox->show_all();
@@ -789,7 +793,7 @@ void MAHealthPanel::add_group(GraphPage *page, MYX_HEALTH_GROUP *group)
 
   Gtk::EventBox *evbox= Gtk::manage(new Gtk::EventBox);
   evbox->add(*frame);
-  evbox->signal_button_press_event().connect(SigC::bind<GraphGroup*>(SigC::slot(*this,&MAHealthPanel::group_button_press),
+  evbox->signal_button_press_event().connect(sigc::bind<GraphGroup*>(sigc::mem_fun(*this,&MAHealthPanel::group_button_press),
                                                                      group_info));
   page->box->pack_start(*evbox, false, true);
 
@@ -818,7 +822,7 @@ MAHealthPanel::GraphItem *MAHealthPanel::add_graph(GraphGroup *group, MYX_HEALTH
       {
         Gtk::EventBox *evbox= Gtk::manage(new Gtk::EventBox);
         evbox->add(*tg->get_widget());
-        evbox->signal_button_press_event().connect(SigC::bind<GraphItem*>(SigC::slot(*this,&MAHealthPanel::graph_button_press),
+        evbox->signal_button_press_event().connect(sigc::bind<GraphItem*>(sigc::mem_fun(*this,&MAHealthPanel::graph_button_press),
                                                                           item));
         group->box->pack_start(*evbox, true, true);
         item->evbox= evbox;
@@ -839,7 +843,7 @@ MAHealthPanel::GraphItem *MAHealthPanel::add_graph(GraphGroup *group, MYX_HEALTH
       {
         Gtk::EventBox *evbox= Gtk::manage(new Gtk::EventBox);
         evbox->add(*bg->get_widget());
-        evbox->signal_button_press_event().connect(SigC::bind<GraphItem*>(SigC::slot(*this,&MAHealthPanel::graph_button_press),
+        evbox->signal_button_press_event().connect(sigc::bind<GraphItem*>(sigc::mem_fun(*this,&MAHealthPanel::graph_button_press),
                                                                           item));
         group->box->pack_start(*evbox, false, true);
         item->evbox= evbox;
@@ -1194,6 +1198,7 @@ bool MAHealthPanel::commit_graph_edit(MYX_HEALTH_GRAPH *graph)
   {
     Gtk::MessageDialog dlg(*_app->window(),
                            _("Please fill the Value Formula field."),
+                           false,
                            Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK);
     dlg.run();
     return false;
@@ -1202,6 +1207,7 @@ bool MAHealthPanel::commit_graph_edit(MYX_HEALTH_GRAPH *graph)
   {
     Gtk::MessageDialog dlg(*_app->window(),
                            _("The Value Formula has a syntax error."),
+                           false,
                            Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK);
     dlg.run();
 
@@ -1212,6 +1218,7 @@ bool MAHealthPanel::commit_graph_edit(MYX_HEALTH_GRAPH *graph)
   {
     Gtk::MessageDialog dlg(*_app->window(),
                            _("The Max. Formula field has a syntax error."),
+                           false,
                            Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK);
     dlg.run();
     return false;
@@ -1220,6 +1227,7 @@ bool MAHealthPanel::commit_graph_edit(MYX_HEALTH_GRAPH *graph)
   {
     Gtk::MessageDialog dlg(*_app->window(),
                            _("Minimal and Maximal Value fields have inconsistent values (Minimal must be smaller than Maximal)."),
+                           false,
                            Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK);
     dlg.run();
     return false;
@@ -1309,6 +1317,7 @@ void MAHealthPanel::add_graph_mi()
 void MAHealthPanel::delete_graph_mi()
 {
   Gtk::MessageDialog dlg(*_app->window(), _("Please confirm graph removal."),
+                         false,
                          Gtk::MESSAGE_INFO, Gtk::BUTTONS_OK_CANCEL, true);
   if (dlg.run() != Gtk::RESPONSE_OK)
     return;
@@ -1401,6 +1410,7 @@ void MAHealthPanel::add_group_mi()
 void MAHealthPanel::delete_group_mi()
 {
   Gtk::MessageDialog dlg(*_app->window(), _("Please confirm group removal. All graphs inside will be removed."),
+                         false,
                          Gtk::MESSAGE_INFO, Gtk::BUTTONS_OK_CANCEL, true);
   if (dlg.run() != Gtk::RESPONSE_OK)
     return;
@@ -1513,6 +1523,7 @@ void MAHealthPanel::add_page_mi()
 void MAHealthPanel::delete_page_mi()
 {
   Gtk::MessageDialog dlg(*_app->window(), _("Please confirm page removal. All groups and graphs inside will be removed."),
+                         false,
                          Gtk::MESSAGE_INFO, Gtk::BUTTONS_OK_CANCEL, true);
   if (dlg.run()!=Gtk::RESPONSE_OK)
     return;
