@@ -105,7 +105,7 @@ void MARestorePanel::setup_selection_list()
   column= new Gtk::TreeView::Column(_("Objects"), *Gtk::manage(trend));
   tree->append_column(*Gtk::manage(column));
   column->add_attribute(trend->property_state(), _sel_columns._state);
-  trend->signal_toggled().connect(SigC::slot(*this,&MARestorePanel::row_toggled));
+  trend->signal_toggled().connect(sigc::mem_fun(*this,&MARestorePanel::row_toggled));
 
   column= tree->get_column(0);
   column->pack_start(_sel_columns._icon, false);
@@ -117,7 +117,7 @@ void MARestorePanel::setup_selection_list()
 
   Glib::RefPtr<Gtk::TreeSelection> sel= tree->get_selection();
 
-//  sel->signal_changed().connect(SigC::slot(*this,&MARestorePanel::selected_object));
+//  sel->signal_changed().connect(sigc::mem_fun(*this,&MARestorePanel::selected_object));
 
 }
 
@@ -133,8 +133,8 @@ bool MARestorePanel::init()
 
   _progress_xml= new MGGladeXML(get_glade_file(GLADE_RESTORE_FILE), "progress_dialog");
 
-  ((Gtk::Button*)_progress_xml->get_widget("stop_button"))->signal_clicked().connect(SigC::slot(*this,&MARestorePanel::abort_operation));
-  ((Gtk::Button*)_progress_xml->get_widget("ok_button"))->signal_clicked().connect(SigC::slot(*this,&MARestorePanel::close_progress));
+  ((Gtk::Button*)_progress_xml->get_widget("stop_button"))->signal_clicked().connect(sigc::mem_fun(*this,&MARestorePanel::abort_operation));
+  ((Gtk::Button*)_progress_xml->get_widget("ok_button"))->signal_clicked().connect(sigc::mem_fun(*this,&MARestorePanel::close_progress));
 
   _progress= (Gtk::ProgressBar*)_progress_xml->get_widget("progressbar");
   
@@ -145,15 +145,17 @@ bool MARestorePanel::init()
 
   _table_icon= PIXCACHE->load("16x16_Table.png");
   _column_icon= PIXCACHE->load("16x16_Field.png");
+  _view_icon= PIXCACHE->load("16x16_View.png");
+  _proc_icon= PIXCACHE->load("16x16_StoredProc.png");
 
   _browser= new MGFileBrowserList(_("Backup File to Restore"),
                                   prefs.last_backup_directory,
                                   true);
   _browser->set_extension(".sql");
 
-  _browser->signal_selected().connect(SigC::slot(*this,&MARestorePanel::file_selected));
+  _browser->signal_selected().connect(sigc::mem_fun(*this,&MARestorePanel::file_selected));
 
-  get_button("restore_button")->signal_clicked().connect(SigC::slot(*this,&MARestorePanel::perform_restore));
+  get_button("restore_button")->signal_clicked().connect(sigc::mem_fun(*this,&MARestorePanel::perform_restore));
 
   _panel->set_sensitive(false);
   
@@ -203,10 +205,22 @@ void MARestorePanel::show_content_info(const std::string &file,
     tit= _selection_store->append(row.children());
     trow= *tit;
 
-    trow[_sel_columns._icon]= _table_icon;
+    if(table->flags & (MYX_BTF_IS_PROCEDURE|MYX_BTF_IS_FUNCTION))
+    {
+      trow[_sel_columns._icon]= _proc_icon;
+    }
+    else if(table->flags & MYX_BTF_IS_VIEW)
+    {
+      trow[_sel_columns._icon]= _view_icon;
+    }
+    else
+    {
+      trow[_sel_columns._icon]= _table_icon;
+    }
+    
     trow[_sel_columns._state]= MGCellRendererTristate::ON;
-	trow[_sel_columns._object]= (char*)table->table;
-	
+    trow[_sel_columns._object]= (char*)table->table;
+
 //    trow[_sel_columns._type]= "";//XXX
 //    trow[_sel_columns._rows]= 0;
     trow[_sel_columns._what]= SelectionColumns::CTable;
@@ -324,28 +338,6 @@ void MARestorePanel::row_toggled(const Glib::ustring &path)
   }
 }
 
-
-bool MARestorePanel::detect_charset(Glib::ustring &chset)
-{
-  std::string item, path;
-
-  _browser->get_row_object(_browser->get_selected(), item);
-
-  path= _browser->get_path_for_entry(item);
-
-  char *charset= myx_detect_charset_from_sql_file(path.c_str());
-  if (charset)
-  {
-    chset= charset;
-
-    g_free(charset);
-    return true;
-  }
-  return false;
-}
-
-
-
 void MARestorePanel::file_selected(MGBrowserList *sender, const Gtk::TreeIter &node)
 {  
   if (node)
@@ -365,11 +357,9 @@ void MARestorePanel::file_selected(MGBrowserList *sender, const Gtk::TreeIter &n
     }
 
     Glib::ustring tmp;
-    if (!detect_charset(tmp))
     {
       MGCharsetPickDialog dialog;
-      
-      dialog.set_detect_function(SigC::slot(*this,&MARestorePanel::detect_charset));
+
       charset= dialog.show(_("Pick the character set used in the selected file."));
       if (charset.empty())
       {
@@ -378,8 +368,6 @@ void MARestorePanel::file_selected(MGBrowserList *sender, const Gtk::TreeIter &n
       }
       dialog.hide();
     }
-    else
-      charset= tmp;
 
     _app->push_status(_("Processing backup file..."));
     _cur_content= perform_load(path, charset);
@@ -392,6 +380,7 @@ void MARestorePanel::file_selected(MGBrowserList *sender, const Gtk::TreeIter &n
         Gtk::MessageDialog dlg(*_app->window(),
                                ufmt(_("Error reading backup file\n'%s'"),
                                     path.c_str()),
+                               false,
                                Gtk::MESSAGE_ERROR,
                                Gtk::BUTTONS_OK,
                                true);
@@ -582,8 +571,8 @@ MYX_BACKUP_CONTENT *MARestorePanel::perform_load(const std::string &file,
   op_aborted= false;
 
   {
-    SigC::Connection con= Glib::signal_timeout().connect(SigC::bind<void*>
-                                                         (SigC::slot(*this, &MARestorePanel::update_status),&arg.pdata),
+    SigC::Connection con= Glib::signal_timeout().connect(sigc::bind<void*>
+                                                         (sigc::mem_fun(*this, &MARestorePanel::update_status),&arg.pdata),
                                                          50);
 
     dlg->show();
@@ -633,10 +622,9 @@ bool MARestorePanel::check_tables_to_be_dropped(MYX_BACKUP_CONTENT *content)
   
   myx_free_backup_content(drop_list);
   
-  Gtk::MessageDialog dlg(Glib::ustring(_("The following tables in the backup file already exist and will be dropped before restoring from backup.\n"
-                                         "Would you like to drop these tables and proceed with restoration?"))+"\n"+msg,
-                         Gtk::MESSAGE_WARNING,
-                         Gtk::BUTTONS_YES_NO);
+  msg= _("The following tables in the backup file already exist and will be dropped before restoring from backup.\nWould you like to drop these tables and proceed with restoration?");
+  msg= "\n"+msg;  
+  Gtk::MessageDialog dlg(msg, false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_YES_NO);
 
   if (dlg.run()==Gtk::RESPONSE_NO)
   {
@@ -690,8 +678,8 @@ void MARestorePanel::perform_restore()
   //{
     MYX_BACKUP_ERROR err;
 
-    SigC::Connection con= Glib::signal_timeout().connect(SigC::bind<void*>
-                                                         (SigC::slot(*this, &MARestorePanel::update_status),&arg.pdata),
+    SigC::Connection con= Glib::signal_timeout().connect(sigc::bind<void*>
+                                                         (sigc::mem_fun(*this, &MARestorePanel::update_status),&arg.pdata),
                                                          50);
 
     dlg->show();
@@ -708,9 +696,10 @@ void MARestorePanel::perform_restore()
   {
     Glib::ustring msg= MABackupPanel::get_backup_error_text(_data->get_instance()->get_mysql(), err);
     Gtk::MessageDialog errdlg(*_app->window(), _("Error during backup.\n")+msg,
-                           Gtk::MESSAGE_ERROR,
-                           Gtk::BUTTONS_OK,
-                           true);
+                              false,
+                              Gtk::MESSAGE_ERROR,
+                              Gtk::BUTTONS_OK,
+                              true);
     errdlg.run();
     _app->set_status(_("Restore failure."));
     dlg->hide();
