@@ -21,14 +21,46 @@
 #include "MAUserAdministrationPanel.h"
 
 #include "MGUserBrowserList.h"
-#include "MGSchemaBrowserList.h"
 #include "MGPreferencesEditor.h"
 
 #include "myg_utils.h"
 #include "myg_gtk_utils.h"
 #include "myg_gtkutils.h"
 
-#include "myx_aux_functions.h"
+#include "myx_util_functions.h"
+
+// first call super::refresh_list
+// then add schemas from MYX_USER that do not exist in the tree
+void MGUserSchemaBrowserList::refresh_list(const Glib::ustring &filter)
+{
+  MYX_USER * user = _panel.get_current_user();
+  //const char * uname = user ? (char *)user->user_name : "";
+
+  MGSchemaBrowserList::refresh_list(filter);  
+  
+  if(user)
+  {
+    for(unsigned int i = 0; i < user->user_object_privileges_num; i++)
+    {
+      const char *oname= (const char *)user->user_object_privileges[i].object_name;
+      if(strlen(oname) == 0)
+      {
+        continue;  
+      }
+      MYX_SCHEMA * sch= new MYX_SCHEMA;
+      sch->schema_name= g_strdup(oname);
+      sch->catalog_name= NULL;
+      if(!contains_schema(sch))
+      {
+        append_schema(sch);
+      }
+      else
+      {
+        delete sch;
+      }
+    }
+  }  
+}
 
 
 MAUserAdministrationPanel::MAUserAdministrationPanel(MAdministrator *app, MDataInterface *data)
@@ -36,14 +68,17 @@ MAUserAdministrationPanel::MAUserAdministrationPanel(MAdministrator *app, MDataI
       _current_user(0), _current_user_icon(0), _current_icon_data(0),
       _global_privileges_template(0),
       _schema_privileges_template(0), _table_privileges_template(0),
-      _column_privileges_template(0),
+      _column_privileges_template(0), _proc_privileges_template(0),
       _user_browser(0), _schema_browser(0), _table_browser(0),
       _first_show(true), _dirty(false), 
       _ignore_entry_changes(false), _ignore_limit_changes(false)
 {
   _instance= data->get_instance();
-  
-  _instance->signal_disconnect().connect(SigC::slot(*this,&MAUserAdministrationPanel::disconnected_cb));
+
+  _apply_changes_on_user_switch= false;
+  _discard_changes_on_user_switch= false;
+
+  _instance->signal_disconnect().connect(sigc::mem_fun(*this,&MAUserAdministrationPanel::disconnected_cb));
 }
 
 
@@ -76,7 +111,7 @@ void MAUserAdministrationPanel::setup_list_views()
   //--
   _global_assigned_privs= Gtk::ListStore::create(_assigned_columns);
   _global_assigned_privs->set_sort_func(_assigned_columns._text,
-                                        SigC::slot(*this, &MAUserAdministrationPanel::assigned_list_comparer));
+                                        sigc::mem_fun(*this, &MAUserAdministrationPanel::assigned_list_comparer));
 
   tree= (Gtk::TreeView*)get_widget("global_assigned_list");
   tree->set_model(_global_assigned_privs);
@@ -86,14 +121,14 @@ void MAUserAdministrationPanel::setup_list_views()
 
   sel= tree->get_selection();
   sel->set_mode(Gtk::SELECTION_MULTIPLE);
-  sel->signal_changed().connect(SigC::bind<Glib::RefPtr<Gtk::TreeSelection>,bool>
-                                (SigC::slot(*this, &MAUserAdministrationPanel::global_privilege_list_selected),
+  sel->signal_changed().connect(sigc::bind<Glib::RefPtr<Gtk::TreeSelection>,bool>
+                                (sigc::mem_fun(*this, &MAUserAdministrationPanel::global_privilege_list_selected),
                                  sel, true));
 
   //--
   _global_available_privs= Gtk::ListStore::create(_available_columns);
   _global_available_privs->set_sort_func(_available_columns._text,
-                                         SigC::slot(*this, &MAUserAdministrationPanel::available_list_comparer));
+                                         sigc::mem_fun(*this, &MAUserAdministrationPanel::available_list_comparer));
 
   tree= (Gtk::TreeView*)get_widget("global_available_list");
   tree->set_model(_global_available_privs);
@@ -104,8 +139,8 @@ void MAUserAdministrationPanel::setup_list_views()
 
   sel= tree->get_selection();
   sel->set_mode(Gtk::SELECTION_MULTIPLE);
-  sel->signal_changed().connect(SigC::bind<Glib::RefPtr<Gtk::TreeSelection>,bool>
-                                (SigC::slot(*this, &MAUserAdministrationPanel::global_privilege_list_selected),
+  sel->signal_changed().connect(sigc::bind<Glib::RefPtr<Gtk::TreeSelection>,bool>
+                                (sigc::mem_fun(*this, &MAUserAdministrationPanel::global_privilege_list_selected),
                                  sel, false));
 
   // ** setup schema priv tables
@@ -113,7 +148,7 @@ void MAUserAdministrationPanel::setup_list_views()
   //--
   _schema_assigned_privs= Gtk::ListStore::create(_assigned_columns);
   _schema_assigned_privs->set_sort_func(_assigned_columns._text,
-                                        SigC::slot(*this, &MAUserAdministrationPanel::assigned_list_comparer));
+                                        sigc::mem_fun(*this, &MAUserAdministrationPanel::assigned_list_comparer));
 
   tree= (Gtk::TreeView*)get_widget("schema_assigned_list");
   tree->set_model(_schema_assigned_privs);
@@ -128,14 +163,14 @@ void MAUserAdministrationPanel::setup_list_views()
 
   sel= tree->get_selection();
   sel->set_mode(Gtk::SELECTION_MULTIPLE);
-  sel->signal_changed().connect(SigC::bind<Glib::RefPtr<Gtk::TreeSelection>,bool>
-                                (SigC::slot(*this, &MAUserAdministrationPanel::schema_privilege_list_selected),
+  sel->signal_changed().connect(sigc::bind<Glib::RefPtr<Gtk::TreeSelection>,bool>
+                                (sigc::mem_fun(*this, &MAUserAdministrationPanel::schema_privilege_list_selected),
                                  sel, true));
 
   //--
   _schema_available_privs= Gtk::ListStore::create(_available_columns);
   _schema_available_privs->set_sort_func(_available_columns._text,
-                                         SigC::slot(*this, &MAUserAdministrationPanel::available_list_comparer));
+                                         sigc::mem_fun(*this, &MAUserAdministrationPanel::available_list_comparer));
 
   tree= (Gtk::TreeView*)get_widget("schema_available_list");
   tree->set_model(_schema_available_privs);
@@ -146,8 +181,8 @@ void MAUserAdministrationPanel::setup_list_views()
 
   sel= tree->get_selection();
   sel->set_mode(Gtk::SELECTION_MULTIPLE);
-  sel->signal_changed().connect(SigC::bind<Glib::RefPtr<Gtk::TreeSelection>,bool>
-                                (SigC::slot(*this, &MAUserAdministrationPanel::schema_privilege_list_selected),
+  sel->signal_changed().connect(sigc::bind<Glib::RefPtr<Gtk::TreeSelection>,bool>
+                                (sigc::mem_fun(*this, &MAUserAdministrationPanel::schema_privilege_list_selected),
                                  sel, false));
 
   // ** setup table/column priv tables
@@ -155,7 +190,7 @@ void MAUserAdministrationPanel::setup_list_views()
   //--
   _table_assigned_privs= Gtk::ListStore::create(_assigned_columns);
   _table_assigned_privs->set_sort_func(_assigned_columns._text,
-                                        SigC::slot(*this, &MAUserAdministrationPanel::assigned_list_comparer));
+                                        sigc::mem_fun(*this, &MAUserAdministrationPanel::assigned_list_comparer));
 
   tree= (Gtk::TreeView*)get_widget("table_assigned_list");
   tree->set_model(_table_assigned_privs);
@@ -165,14 +200,14 @@ void MAUserAdministrationPanel::setup_list_views()
 
   sel= tree->get_selection();
   sel->set_mode(Gtk::SELECTION_MULTIPLE);
-  sel->signal_changed().connect(SigC::bind<Glib::RefPtr<Gtk::TreeSelection>,bool>
-                                (SigC::slot(*this, &MAUserAdministrationPanel::table_privilege_list_selected),
+  sel->signal_changed().connect(sigc::bind<Glib::RefPtr<Gtk::TreeSelection>,bool>
+                                (sigc::mem_fun(*this, &MAUserAdministrationPanel::table_privilege_list_selected),
                                  sel, true));
 
   //--
   _table_available_privs= Gtk::ListStore::create(_available_columns);
   _table_available_privs->set_sort_func(_available_columns._text,
-                                        SigC::slot(*this, &MAUserAdministrationPanel::available_list_comparer));
+                                        sigc::mem_fun(*this, &MAUserAdministrationPanel::available_list_comparer));
 
   tree= (Gtk::TreeView*)get_widget("table_available_list");
   tree->set_model(_table_available_privs);
@@ -183,8 +218,8 @@ void MAUserAdministrationPanel::setup_list_views()
 
   sel= tree->get_selection();
   sel->set_mode(Gtk::SELECTION_MULTIPLE);
-  sel->signal_changed().connect(SigC::bind<Glib::RefPtr<Gtk::TreeSelection>,bool>
-                                (SigC::slot(*this, &MAUserAdministrationPanel::table_privilege_list_selected),
+  sel->signal_changed().connect(sigc::bind<Glib::RefPtr<Gtk::TreeSelection>,bool>
+                                (sigc::mem_fun(*this, &MAUserAdministrationPanel::table_privilege_list_selected),
                                  sel, false));
 }
 
@@ -205,13 +240,13 @@ static Gtk::MenuItem *make_item(Gtk::Menu *menu,
 void MAUserAdministrationPanel::connect_signals()
 {
 #define CONNECT_CLICKED(name, cb) \
-  ((Gtk::Button*)_xml->get_widget(name))->signal_clicked().connect(SigC::slot(*this, &MAUserAdministrationPanel::cb))
+  ((Gtk::Button*)_xml->get_widget(name))->signal_clicked().connect(sigc::mem_fun(*this, &MAUserAdministrationPanel::cb))
 
 #define CONNECT_CLICKEDB(name, cb, val) \
-  ((Gtk::Button*)_xml->get_widget(name))->signal_clicked().connect(SigC::bind<bool>(SigC::slot(*this, &MAUserAdministrationPanel::cb),val))
+  ((Gtk::Button*)_xml->get_widget(name))->signal_clicked().connect(sigc::bind<bool>(sigc::mem_fun(*this, &MAUserAdministrationPanel::cb),val))
 
 #define CONNECT_CHANGED(name, cb) \
-  ((Gtk::Entry*)_xml->get_widget(name))->signal_changed().connect(SigC::slot(*this, &MAUserAdministrationPanel::cb))
+  ((Gtk::Entry*)_xml->get_widget(name))->signal_changed().connect(sigc::mem_fun(*this, &MAUserAdministrationPanel::cb))
 
   CONNECT_CLICKEDB("global_grant_button", global_revoke_grant_button_clicked,
                    false);
@@ -234,7 +269,7 @@ void MAUserAdministrationPanel::connect_signals()
   CONNECT_CLICKED("discard_button", discard_button_clicked);
   CONNECT_CLICKED("new_user_button", new_user_button_clicked);
  
-  ((Gtk::TextView*)get_widget("contact_text"))->get_buffer()->signal_changed().connect(SigC::slot(*this,&MAUserAdministrationPanel::entry_changed));
+  ((Gtk::TextView*)get_widget("contact_text"))->get_buffer()->signal_changed().connect(sigc::mem_fun(*this,&MAUserAdministrationPanel::entry_changed));
   CONNECT_CHANGED("user_entry", entry_changed);
   CONNECT_CHANGED("password_entry", entry_changed);
   CONNECT_CHANGED("confirm_entry", entry_changed);
@@ -242,12 +277,12 @@ void MAUserAdministrationPanel::connect_signals()
   CONNECT_CHANGED("description_entry", entry_changed);
   CONNECT_CHANGED("email_entry", entry_changed);
 
-  get_spin("maxq_spin")->signal_changed().connect(SigC::bind<const char*>
-                                                  (SigC::slot(*this,&MAUserAdministrationPanel::limit_changed),"maxq_spin"));
-  get_spin("maxc_spin")->signal_changed().connect(SigC::bind<const char*>
-                                                  (SigC::slot(*this,&MAUserAdministrationPanel::limit_changed),"maxc_spin"));
-  get_spin("maxu_spin")->signal_changed().connect(SigC::bind<const char*>
-                                                  (SigC::slot(*this,&MAUserAdministrationPanel::limit_changed),"maxu_spin"));
+  get_spin("maxq_spin")->signal_changed().connect(sigc::bind<const char*>
+                                                  (sigc::mem_fun(*this,&MAUserAdministrationPanel::limit_changed),"maxq_spin"));
+  get_spin("maxc_spin")->signal_changed().connect(sigc::bind<const char*>
+                                                  (sigc::mem_fun(*this,&MAUserAdministrationPanel::limit_changed),"maxc_spin"));
+  get_spin("maxu_spin")->signal_changed().connect(sigc::bind<const char*>
+                                                  (sigc::mem_fun(*this,&MAUserAdministrationPanel::limit_changed),"maxu_spin"));
 }
 
 
@@ -257,6 +292,7 @@ bool MAUserAdministrationPanel::init()
   Glib::RefPtr<Gdk::Pixbuf> sys_schema_icon;
   Glib::RefPtr<Gdk::Pixbuf> table_icon;
   Glib::RefPtr<Gdk::Pixbuf> column_icon;
+  Glib::RefPtr<Gdk::Pixbuf> sp_icon;
 
   if (_xml)
     return true;
@@ -266,10 +302,10 @@ bool MAUserAdministrationPanel::init()
     return false;
 
   _dialog_xml= new MGGladeXML(get_glade_file(GLADE_USERADMINISTRATION_FILE),
-                              "icon_selection");
+                              "icon_selection", "mysql-administrator");
 
   _host_dialog_xml= new MGGladeXML(get_glade_file(GLADE_USERADMINISTRATION_FILE),
-                                   "add_host_dialog");
+                                   "add_host_dialog", "mysql-administrator");
 
   
   _size_group= Gtk::SizeGroup::create(Gtk::SIZE_GROUP_HORIZONTAL);
@@ -290,11 +326,11 @@ bool MAUserAdministrationPanel::init()
   
     item= make_item(_menu, _("_New User"), Gtk::Stock::NEW);
     _user_menu_items[i++]= item;
-    item->signal_activate().connect(SigC::slot(*this, &MAUserAdministrationPanel::add_user_activate));
+    item->signal_activate().connect(sigc::mem_fun(*this, &MAUserAdministrationPanel::add_user_activate));
 
 /*    item= make_item(_menu, _("_Clone User"), Gtk::Stock::COPY);
     _user_menu_items[i++]= item;
-    item->signal_activate().connect(SigC::slot(*this, &MAUserAdministrationPanel::clone_user_activate));
+    item->signal_activate().connect(sigc::mem_fun(*this, &MAUserAdministrationPanel::clone_user_activate));
 */
     item= new Gtk::SeparatorMenuItem();
     item->show();
@@ -302,11 +338,11 @@ bool MAUserAdministrationPanel::init()
 
     item= make_item(_menu, _("Add _Host"), Gtk::Stock::ADD);
     _user_menu_items[i++]= item;
-    item->signal_activate().connect(SigC::slot(*this, &MAUserAdministrationPanel::add_host_activate));
+    item->signal_activate().connect(sigc::mem_fun(*this, &MAUserAdministrationPanel::add_host_activate));
 
     item= make_item(_menu, _("_Remove Host"), Gtk::Stock::REMOVE);
     _user_menu_items[i++]= item;
-    item->signal_activate().connect(SigC::slot(*this, &MAUserAdministrationPanel::remove_host_activate));
+    item->signal_activate().connect(sigc::mem_fun(*this, &MAUserAdministrationPanel::remove_host_activate));
 
     item= new Gtk::SeparatorMenuItem();
     item->show();
@@ -314,7 +350,7 @@ bool MAUserAdministrationPanel::init()
 
     item= make_item(_menu, _("R_emove User"), Gtk::Stock::DELETE);
     _user_menu_items[i++]= item;
-    item->signal_activate().connect(SigC::slot(*this, &MAUserAdministrationPanel::remove_user_activate));
+    item->signal_activate().connect(sigc::mem_fun(*this, &MAUserAdministrationPanel::remove_user_activate));
 
     
 //    item= new Gtk::SeparatorMenuItem();
@@ -324,7 +360,7 @@ bool MAUserAdministrationPanel::init()
     //XXX
 //    item= make_item(_menu, _("Refresh list"), Gtk::Stock::REFRESH);
   //  _user_menu_items[i++]= item;
-//    item->signal_activate().connect(SigC::slot(*this, &MAUserAdministrationPanel::toggle_advanced));
+//    item->signal_activate().connect(sigc::mem_fun(*this, &MAUserAdministrationPanel::toggle_advanced));
   }
 
   _default_user_icon= PIXCACHE->load("24x24_User.png");
@@ -347,40 +383,60 @@ bool MAUserAdministrationPanel::init()
 
   table_icon= PIXCACHE->load("16x16_Table.png");
   column_icon= PIXCACHE->load("16x16_Field.png");
+  sp_icon= PIXCACHE->load("myx_schema_sp_16x16.png");
 
   _user_browser= new MGUserBrowserList(_("User Accounts"));
   _user_browser->set_default_icon(_default_user_mini_icon);
-  _user_browser->signal_selected().connect(SigC::slot(*this,&MAUserAdministrationPanel::user_selected));
-  _user_browser->set_populate_func(SigC::slot(*this,&MAUserAdministrationPanel::populate_user_host));
-  _user_browser->signal_selection_will_change().connect(SigC::slot(*this,&MAUserAdministrationPanel::user_will_change_delegate));
+  _user_browser->signal_selected().connect(sigc::mem_fun(*this,&MAUserAdministrationPanel::user_selected));
+  _user_browser->set_populate_func(sigc::mem_fun(*this,&MAUserAdministrationPanel::populate_user_host));
+  _user_browser->signal_selection_will_change().connect(sigc::mem_fun(*this,&MAUserAdministrationPanel::user_will_change_delegate));
 
   _user_browser->set_popup_menu(_menu);
 
-  _schema_browser= new MGSchemaBrowserList("", MGSchemaBrowserList::SchemaOnly);
-  _schema_browser->signal_selected().connect(SigC::slot(*this,&MAUserAdministrationPanel::schema_selected));
+  _schema_browser= new MGUserSchemaBrowserList(*this, "", MGSchemaBrowserList::SchemaOnly);
+  _schema_browser->signal_selected().connect(sigc::mem_fun(*this,&MAUserAdministrationPanel::schema_selected));
   _schema_browser->set_mark_delegate(schema_mark_delegate, this);
-  _schema_browser->set_icons(schema_icon, sys_schema_icon, table_icon, column_icon);
+  _schema_browser->set_icons(schema_icon, sys_schema_icon, table_icon, column_icon, sp_icon);
   ((Gtk::Container*)get_widget("schema_frame"))->add(*Gtk::manage(_schema_browser));
   _schema_browser->show();
   
   _table_browser= new MGSchemaBrowserList("", MGSchemaBrowserList::Normal);
-  _table_browser->signal_selected().connect(SigC::slot(*this,&MAUserAdministrationPanel::table_selected));
-  _table_browser->set_populate_func(SigC::slot(*this,&MAUserAdministrationPanel::populate_schema_tables));
+  _table_browser->signal_selected().connect(sigc::mem_fun(*this,&MAUserAdministrationPanel::table_selected));
+  _table_browser->set_populate_func(sigc::mem_fun(*this,&MAUserAdministrationPanel::populate_schema_tables));
   _table_browser->set_mark_delegate(table_mark_delegate, this);
-  _table_browser->set_icons(schema_icon, sys_schema_icon, table_icon, column_icon);
+  _table_browser->set_icons(schema_icon, sys_schema_icon, table_icon, column_icon, sp_icon);
   ((Gtk::Container*)get_widget("table_frame"))->add(*Gtk::manage(_table_browser));
   _table_browser->show();
 
   setup_list_views();
 
   connect_signals();
-  
+
+  {
+    Gtk::MenuItem *item;
+    
+    _schemas_popup_menu= new Gtk::Menu();
+    item= make_item(_menu, _("_Remove Schema from Priv List"), Gtk::Stock::NEW);
+    //_user_menu_items[i++]= item;
+    //item->signal_activate().connect(sigc::mem_fun(*this, &MAUserAdministrationPanel::add_user_activate));
+    _schema_browser->set_popup_menu(_schemas_popup_menu);    
+  }
+
   // monitor changes in preferences
-  _app->signal_prefs_changed().connect(SigC::slot(*this,&MAUserAdministrationPanel::prefs_changed));
+  _app->signal_prefs_changed().connect(sigc::mem_fun(*this,&MAUserAdministrationPanel::prefs_changed));
 
   return true;
 }
 
+void MAUserAdministrationPanel::reload_catalogs()
+{
+  Glib::RefPtr<MGPtrWrap<MYX_CATALOGS*> > catalogs= _data->get_catalogs();
+  if (catalogs && catalogs->ptr())
+  {
+    _schema_browser->set_catalogs(catalogs->ptr());
+    _table_browser->set_catalogs(catalogs->ptr());
+  }
+}
 
 void MAUserAdministrationPanel::show()
 {
@@ -401,12 +457,15 @@ void MAUserAdministrationPanel::show()
   if (_user_names)
     _user_browser->set_user_list(_user_names);
 
+  /*
   Glib::RefPtr<MGPtrWrap<MYX_CATALOGS*> > catalogs= _data->get_catalogs();
   if (catalogs && catalogs->ptr())
   {
     _schema_browser->set_catalogs(catalogs->ptr());
     _table_browser->set_catalogs(catalogs->ptr());
   }
+  */
+  reload_catalogs();
 }
 
 
@@ -510,6 +569,7 @@ typedef struct {
   MYX_USER_OBJECT_PRIVILEGES *global;
   MYX_USER_OBJECT_PRIVILEGES *schema;
   MYX_USER_OBJECT_PRIVILEGES *table;
+  MYX_USER_OBJECT_PRIVILEGES *proc;
   MYX_USER_OBJECT_PRIVILEGES *column;
 } WantedPrivileges;
 
@@ -520,16 +580,21 @@ static void* fetch_privilege_templates_worker(MYSQL *mysql)
   
   if (!(privs->global= myx_get_privilege_struct(mysql, "")))
     goto end;
-
   if (!(privs->schema= myx_get_privilege_struct(mysql, "x")))
     goto end;
-
-  if (!(privs->table= myx_get_privilege_struct(mysql, "x.x")))
+  if (mysql_version_is_later_or_equal_than(mysql,5,0))
+  {
+    if (!(privs->proc= myx_get_privilege_struct(mysql, "x.x")))
+      goto end;
+  }
+  else
+  {
+    privs->proc= NULL;
+  }    
+  if (!(privs->table= myx_get_privilege_struct(mysql, "mysql.user")))
     goto end;
-  
   if (!(privs->column= myx_get_privilege_struct(mysql, "x.x.x")))
     goto end;
-
   return privs;
 end:
   g_free(privs->global);
@@ -546,7 +611,6 @@ bool MAUserAdministrationPanel::fetch_privilege_templates()
   WantedPrivileges *privs;
   
   privs= (WantedPrivileges*)_instance->perform_data_fetch((MInstanceInfo::DataFetcher)fetch_privilege_templates_worker);
-  
   if (privs)
   {
     _global_privileges_template= privs->global;
@@ -555,6 +619,8 @@ bool MAUserAdministrationPanel::fetch_privilege_templates()
     
     _table_privileges_template= privs->table;
     
+    _proc_privileges_template= privs->proc;
+    
     _column_privileges_template= privs->column;
     
     g_free(privs);
@@ -568,7 +634,8 @@ MYX_USER_OBJECT_PRIVILEGES *
   MAUserAdministrationPanel::fetch_privileges_for_object(MYX_USER *user,
                                                          const Glib::ustring &host,
                                                          const Glib::ustring &object,
-                                                         bool cache_only)
+                                                         bool cache_only,
+                                                         bool is_stored_proc)
 {
   MYX_USER_OBJECT_PRIVILEGES *priv= 0;
   unsigned int i;
@@ -611,7 +678,14 @@ MYX_USER_OBJECT_PRIVILEGES *
         priv= clone_privilege_struct(_schema_privileges_template);
         break;
       case 1:
-        priv= clone_privilege_struct(_table_privileges_template);
+        if(is_stored_proc) 
+        {
+          priv= clone_privilege_struct(_proc_privileges_template);
+        }
+        else
+        {
+          priv= clone_privilege_struct(_table_privileges_template);
+        }
         break;
       case 2:
         priv= clone_privilege_struct(_column_privileges_template);
@@ -763,12 +837,15 @@ bool MAUserAdministrationPanel::populate_user_host(MGUserBrowserList *sender,
 bool MAUserAdministrationPanel::populate_schema_tables(MGSchemaBrowserList *sender,
                                                        const Gtk::TreeIter &node)
 {
+  bool b1= false, b2= false;
+  
   Glib::ustring catalog, schema, table, column, child;
 
   sender->get_row_object(node, catalog, schema, table, column, child);
 
-  if (_data->get_schema_tables(catalog, schema))
-    return true;
+  b1= _data->get_schema_tables(catalog, schema);
+  b2= _data->get_schema_sps(catalog, schema);
+  return b1 && b2;
 
   return false;
 }
@@ -894,13 +971,13 @@ void MAUserAdministrationPanel::set_shown_user(MYX_USER *user,
     }
     else
     {
-      new_icon= 0;
+      new_icon.clear();
     }
   }
   else
   {
     ustr= " <i>"+Glib::ustring(_("no user selected"))+"</i>\n";
-    new_icon= 0;
+    new_icon.clear();
   }
 
   // user info
@@ -908,7 +985,7 @@ void MAUserAdministrationPanel::set_shown_user(MYX_USER *user,
   ((Gtk::Label*)get_widget("user_label"))->set_markup(str);
     
 
-  ((Gtk::Image*)get_widget("user_image"))->set(!new_icon.is_null()?new_icon:_default_user_icon);
+  ((Gtk::Image*)get_widget("user_image"))->set(new_icon?new_icon:_default_user_icon);
 
   if (_current_user != user)
   {
@@ -941,7 +1018,7 @@ void MAUserAdministrationPanel::set_shown_user(MYX_USER *user,
   str = uhstr + _("Global privileges assigned to the user");
   ((Gtk::Label*)get_widget("global_label"))->set_markup(str);
 
-  ((Gtk::Image*)get_widget("global_image"))->set(!new_icon.is_null()?new_icon:_default_user_icon);
+  ((Gtk::Image*)get_widget("global_image"))->set(new_icon?new_icon:_default_user_icon);
 
   if (host.empty() || _current_host != host || _current_user != user)
   {
@@ -955,7 +1032,7 @@ void MAUserAdministrationPanel::set_shown_user(MYX_USER *user,
   str= uhstr + std::string(_("Schema (database) privileges assigned to the user"));
   ((Gtk::Label*)get_widget("schema_label"))->set_markup(str);
 
-  ((Gtk::Image*)get_widget("schema_image"))->set(!new_icon.is_null()?new_icon:_default_user_icon);
+  ((Gtk::Image*)get_widget("schema_image"))->set(new_icon?new_icon:_default_user_icon);
 
   if (host.empty() || _current_host != host || _current_user != user)
   {
@@ -974,7 +1051,7 @@ void MAUserAdministrationPanel::set_shown_user(MYX_USER *user,
   str= uhstr + std::string(_("Table and column privileges assigned to the user"));
   ((Gtk::Label*)get_widget("table_label"))->set_markup(str);
 
-  ((Gtk::Image*)get_widget("table_image"))->set(!new_icon.is_null()?new_icon:_default_user_icon);
+  ((Gtk::Image*)get_widget("table_image"))->set(new_icon?new_icon:_default_user_icon);
 
   if (host.empty() || _current_host != host || _current_user != user)
   {
@@ -995,7 +1072,7 @@ void MAUserAdministrationPanel::set_shown_user(MYX_USER *user,
   str= uhstr + std::string(_("Various resource limits for the user"));
   ((Gtk::Label*)get_widget("resource_label"))->set_markup(str);
 
-  ((Gtk::Image*)get_widget("resource_image"))->set(!new_icon.is_null()?new_icon:_default_user_icon);
+  ((Gtk::Image*)get_widget("resource_image"))->set(new_icon?new_icon:_default_user_icon);
   if (host.empty() || _current_host != host || _current_user != user)
   {
     show_limits(user, host);
@@ -1150,9 +1227,9 @@ bool MAUserAdministrationPanel::grant_revoke_privilege(MYX_USER_OBJECT_PRIVILEGE
       old_selected++;
   }
   
-  sel->selected_foreach(SigC::bind<MYX_USER_OBJECT_PRIVILEGES*,bool>
-                        (SigC::slot(*this, &MAUserAdministrationPanel::set_privilege_values),
-                         privs, revoke));
+  sel->selected_foreach_iter(sigc::bind<MYX_USER_OBJECT_PRIVILEGES*,bool>
+                             (sigc::mem_fun(*this, &MAUserAdministrationPanel::set_privilege_values),
+                              privs, revoke));
 
   for (unsigned int i= 0; i < privs->user_privileges_num; i++)
   {
@@ -1209,6 +1286,7 @@ void MAUserAdministrationPanel::set_user_icon_file(const Glib::ustring &filename
   {
     Gtk::MessageDialog dlg(*_app->window(),
                            _("The icon must be in PNG format."),
+                           false,
                            Gtk::MESSAGE_ERROR,
                            Gtk::BUTTONS_OK,
                            true);
@@ -1224,6 +1302,7 @@ void MAUserAdministrationPanel::set_user_icon_file(const Glib::ustring &filename
   {
     Gtk::MessageDialog dlg(*_app->window(),
                            _("Error loading icon"),
+                           false,
                            Gtk::MESSAGE_ERROR,
                            Gtk::BUTTONS_OK, 
                            true);
@@ -1399,6 +1478,38 @@ void MAUserAdministrationPanel::show_table_privileges(MYX_USER *user,
   }
 }
 
+void MAUserAdministrationPanel::show_proc_privileges(MYX_USER *user,
+                                                      const Glib::ustring &catalog,
+                                                      const Glib::ustring &schema,
+                                                      const Glib::ustring &table, 
+                                                      const Glib::ustring &host)
+{
+  MYX_USER_OBJECT_PRIVILEGES *privs= NULL;
+  std::string name(schema);
+  name += "."+table;
+
+  if (!host.empty())
+  {
+    if (user)
+    {
+      privs= fetch_privileges_for_object(user, host, name, false, true);
+    }
+    if (privs)
+    {
+      fill_privilege_list(_table_assigned_privs, _table_available_privs,
+                          (char**)privs->user_privileges,
+                          privs->user_privileges_num, 
+                          NULL, 0,
+                          "Proc_priv_", NULL);
+    }
+  }
+
+  if (!privs)
+  {
+    fill_privilege_list(_table_assigned_privs, _table_available_privs,
+                        NULL, 0, NULL, 0, NULL, NULL);
+  }
+}
 
 void MAUserAdministrationPanel::show_limits(MYX_USER *user, const Glib::ustring &host)
 {
@@ -1477,10 +1588,12 @@ void MAUserAdministrationPanel::create_new_user()
     user->email= (unsigned char*)g_strdup("");
     user->contact_information= (unsigned char*)g_strdup("");
 
-    add_host_to_user(user, "localhost");
+    add_host_to_user(user, "%");
     
     _data->add_new_user(user, new_name);
 
+    _user_browser->clear_filter();
+    _user_browser->refresh();
     _user_browser->set_user_list(_user_names);
 
     _user_browser->select_user(new_name);
@@ -1546,6 +1659,7 @@ void MAUserAdministrationPanel::remove_user()
                               _current_user->user_name,//_current_user_name.c_str(),
                               _instance->get_connection_data().hostname.c_str(),
                               _instance->get_connection_data().port),
+                         false,
                          Gtk::MESSAGE_QUESTION,
                          Gtk::BUTTONS_OK_CANCEL,
                          true);
@@ -1598,7 +1712,8 @@ bool MAUserAdministrationPanel::validate()
 
     // check duplicate name
     uname= ((Gtk::Entry*)get_widget("user_entry"))->get_text();
-    if(_user_browser->exists_user(uname))
+    if(strcmp((const char *)_current_user->user_name, uname.c_str()) && 
+      _user_browser->exists_user(uname))
     {
       msg= _("Duplicate user name, please enter another name.");
       break;
@@ -1608,7 +1723,7 @@ bool MAUserAdministrationPanel::validate()
 
   if (!msg.empty())
   {
-    Gtk::MessageDialog dlg(*_app->window(), msg,
+    Gtk::MessageDialog dlg(*_app->window(), msg, false,
                            Gtk::MESSAGE_ERROR,
                            Gtk::BUTTONS_OK,
                            true);
@@ -1766,13 +1881,9 @@ void MAUserAdministrationPanel::discard_changes()
 void MAUserAdministrationPanel::discard_state_data()
 {
   if (_current_user_icon)
-  {
-    _current_user_icon= 0;
-  }
+    _current_user_icon.clear();
   if (_current_icon_data)
-  {
-    _current_icon_data= 0;
-  }
+    _current_icon_data= NULL;
 
   for (std::map<Glib::ustring,MYX_USER_OBJECT_PRIVILEGES*>::iterator it= _object_privilege.begin();
        it != _object_privilege.end(); ++it)
@@ -1822,7 +1933,7 @@ void MAUserAdministrationPanel::add_host()
   Glib::ustring new_host;
 
   // autoselect the 3rd radio when user types in the host entry
-  _host_dialog_xml->get_entry("new_host_entry")->signal_changed().connect(SigC::bind<bool>(SigC::slot(*_host_dialog_xml->get_toggle("new_host_radio3"),
+  _host_dialog_xml->get_entry("new_host_entry")->signal_changed().connect(sigc::bind<bool>(sigc::mem_fun(*_host_dialog_xml->get_toggle("new_host_radio3"),
                                                                                                       &Gtk::ToggleButton::set_active),true));
   
   if (dlg->run()!=Gtk::RESPONSE_OK)
@@ -1891,6 +2002,7 @@ void MAUserAdministrationPanel::remove_host()
     Gtk::MessageDialog dlg(*_app->window(),
                            ufmt(_("Removing the last host from the users host list will also remove user itself. Please confirm."),
                                 _current_host.c_str()),
+                           false,
                            Gtk::MESSAGE_QUESTION,
                            Gtk::BUTTONS_OK_CANCEL,
                            true);
@@ -1906,6 +2018,7 @@ void MAUserAdministrationPanel::remove_host()
     Gtk::MessageDialog dlg(*_app->window(),
                            ufmt(_("Remove '%s' from the list of hosts the user can connect from?"),
                                 _current_host.c_str()),
+                           false,
                            Gtk::MESSAGE_QUESTION,
                            Gtk::BUTTONS_OK_CANCEL,
                            true);
@@ -1980,6 +2093,15 @@ void MAUserAdministrationPanel::remove_host()
   set_dirty();
 }
 
+MYX_USER * MAUserAdministrationPanel::get_current_user()
+{
+  return _current_user;
+}
+
+Glib::ustring MAUserAdministrationPanel::get_current_host()
+{
+  return _current_host;
+}
 
 /***********************************************************************/
 
@@ -2084,6 +2206,8 @@ void MAUserAdministrationPanel::user_selected(MGBrowserList *sender,
   }
   else
     unshow_user();
+  
+  reload_catalogs();
 }
 
 
@@ -2106,6 +2230,7 @@ void MAUserAdministrationPanel::schema_selected(MGBrowserList *sender,
 void MAUserAdministrationPanel::table_selected(MGBrowserList *sender,
                                                const Gtk::TreeIter &node)
 {
+  char t;
   Glib::ustring catalog;
   Glib::ustring schema;
   Glib::ustring table;
@@ -2114,20 +2239,30 @@ void MAUserAdministrationPanel::table_selected(MGBrowserList *sender,
   Gtk::TreeModel::Row row= *node;
 
   if (node)
-    _table_browser->get_row_object(node, catalog, schema, table, column, child);
+    t= _table_browser->get_row_object(node, catalog, schema, table, column, child);
 
   if (!schema.empty() && table.empty() && column.empty() && child.empty() &&
       row[_table_browser->_columns._populated])
   {
     _table_browser->populate_node(node);
   }
-  
+
   if (!table.empty() && child.empty())
   {
-    // display table (or column) privs
-    show_table_privileges(_current_user,
-                          catalog, schema, table, column,
-                          _current_host);
+    if(t == 'P')
+    {
+      show_proc_privileges(_current_user,
+                            catalog, schema, 
+                            table,
+                            _current_host);
+    }
+    else
+    {
+      // display table (or column) privs
+      show_table_privileges(_current_user,
+                            catalog, schema, table, column,
+                            _current_host);
+    }
   }
 }
 
