@@ -130,7 +130,7 @@ void MAdministrator::populate_sidebar()
 
     // set menu callback
     Gtk::MenuItem *item= (Gtk::MenuItem*)get_widget(sidebar_item_list[i].menu_item);
-    item->signal_activate().connect(SigC::bind<int>(SigC::slot(*this,&MAdministrator::switch_panel_num),i));
+    item->signal_activate().connect(sigc::bind<int>(sigc::mem_fun(*this,&MAdministrator::switch_panel_num),i));
 
     // set icon
     try
@@ -169,7 +169,7 @@ void MAdministrator::setup_sidebar()
   populate_sidebar();
 
   select= _tree->get_selection();
-  select->signal_changed().connect(SigC::slot(*this, &MAdministrator::tree_selection_changed_cb));
+  select->signal_changed().connect(sigc::mem_fun(*this, &MAdministrator::tree_selection_changed_cb));
   select->set_mode(Gtk::SELECTION_BROWSE);
   select->select(Gtk::TreePath("0"));
 }
@@ -210,13 +210,14 @@ bool MAdministrator::init(MInstanceInfo *inst)
   _instance= inst;
   _dataif= new MDataInterface(this, inst);
   
-  _xml= new MGGladeXML(get_app_file(GLADE_MAINWINDOW_FILE));
+  _xml= new MGGladeXML(get_app_file(GLADE_MAINWINDOW_FILE), "", "mysql-administrator");
   if (!_xml)
   {
     Gtk::MessageDialog dlg(ufmt(_("The user interface file (%s) could not be loaded.\n"
                                   "%s was probably not installed correctly."),
                                 get_app_file(GLADE_MAINWINDOW_FILE).c_str(),
                                 PACKAGE),
+                           false,
                            Gtk::MESSAGE_ERROR,
                            Gtk::BUTTONS_OK,
                            true);
@@ -227,16 +228,22 @@ bool MAdministrator::init(MInstanceInfo *inst)
   _win= (Gtk::Window*)get_widget("main_window");
   _tree= (Gtk::TreeView*)get_widget("sidebar_tree");
 
-  _win->signal_delete_event().connect(SigC::slot(*this,&MAdministrator::close_cb));
-  ((Gtk::MenuItem*)get_widget("new_connection1"))->signal_activate().connect(SigC::slot(*this,&MAdministrator::new_connection_cb));
-  ((Gtk::MenuItem*)get_widget("close_connection1"))->signal_activate().connect(SigC::slot(*this,&MAdministrator::close_connection_cb));
-  ((Gtk::MenuItem*)get_widget("quit1"))->signal_activate().connect(SigC::slot(*this,&MAdministrator::menu_quit_cb));
+  // edit menu
+  _xml->get_menu_item("menuitem2")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::update_edit_menu));
+  _xml->get_menu_item("cut1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::cut_cb));
+  _xml->get_menu_item("copy1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::copy_cb));
+  _xml->get_menu_item("paste1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::paste_cb));
+  
+  _win->signal_delete_event().connect(sigc::mem_fun(*this,&MAdministrator::close_cb));
+  _xml->get_menu_item("new_connection1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::new_connection_cb));
+  _xml->get_menu_item("close_connection1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::close_connection_cb));
+  _xml->get_menu_item("quit1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::menu_quit_cb));
 
-//  ((Gtk::MenuItem*)get_widget("sidebar1"))->signal_activate().connect(SigC::slot(*this,&MAdministrator::toggle_sidebar_cb));
-  ((Gtk::MenuItem*)get_widget("contents1"))->signal_activate().connect(SigC::slot(*this,&MAdministrator::open_help));
-  ((Gtk::MenuItem*)get_widget("about1"))->signal_activate().connect(SigC::slot(*this,&MAdministrator::menu_about_cb));
+//  _xml->get_menu_item("sidebar1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::toggle_sidebar_cb));
+  _xml->get_menu_item("contents1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::open_help));
+  _xml->get_menu_item("about1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::menu_about_cb));
 
-  ((Gtk::MenuItem*)get_widget("preferences1"))->signal_activate().connect(SigC::slot(*this,&MAdministrator::menu_preferences_cb));
+  _xml->get_menu_item("preferences1")->signal_activate().connect(sigc::mem_fun(*this,&MAdministrator::menu_preferences_cb));
   
   _progressbar= (Gtk::ProgressBar*)get_widget("status_progress");
 
@@ -244,7 +251,7 @@ bool MAdministrator::init(MInstanceInfo *inst)
   
   setup_sidebar();
 
-  ((Gtk::Button*)_xml->get_widget("status_button"))->signal_clicked().connect(SigC::slot(*this, &MAdministrator::cancel_button_clicked_cb));
+  ((Gtk::Button*)_xml->get_widget("status_button"))->signal_clicked().connect(sigc::mem_fun(*this, &MAdministrator::cancel_button_clicked_cb));
 
   if (!_transl.open(get_app_file("mysqlx_translations_general.xml").c_str(),
                     get_app_file("mysqlx_translations_administrator.xml").c_str(),
@@ -253,8 +260,8 @@ bool MAdministrator::init(MInstanceInfo *inst)
     g_warning(_("Could not open translation files"));
   }
 
-  _instance->signal_disconnect().connect(SigC::slot(*this,&MAdministrator::disconnect_db_handler));
-  _instance->signal_connect().connect(SigC::slot(*this,&MAdministrator::connect_db_handler));
+  _instance->signal_disconnect().connect(sigc::mem_fun(*this,&MAdministrator::disconnect_db_handler));
+  _instance->signal_connect().connect(sigc::mem_fun(*this,&MAdministrator::connect_db_handler));
 
   // set window title
   if (_instance->is_connected())
@@ -601,7 +608,7 @@ void MAdministrator::set_status(const Glib::ustring &text)
 }
 
 
-void MAdministrator::push_stop_button_handler(const SigC::Slot0<void> &slot)
+void MAdministrator::push_stop_button_handler(const sigc::slot0<void> &slot)
 {
   _stop_button_handlers.push_back(slot);
 }
@@ -656,8 +663,11 @@ void MAdministrator::set_busy(const Glib::ustring &message)
 void MAdministrator::set_busy_progress(bool start)
 {
   if (start)
-    _pulse_conn= Glib::signal_timeout().connect(SigC::slot(*this, &MAdministrator::pulse_progress),
-                                                10000);
+  {
+    if (!_pulse_conn)
+      _pulse_conn= Glib::signal_timeout().connect(sigc::mem_fun(*this, &MAdministrator::pulse_progress),
+                                                  10000);
+  }
   else
   {
     if (_pulse_conn)
@@ -727,7 +737,7 @@ void MAdministrator::menu_preferences_cb()
   if (!_prefs_editor)
   {
     _prefs_editor= MGPreferencesEditor::instance();
-    _prefs_editor->signal_changed().connect(SigC::slot(*this,&MAdministrator::prefs_changed));
+    _prefs_editor->signal_changed().connect(sigc::mem_fun(*this,&MAdministrator::prefs_changed));
   }
 
   _prefs_editor->show();
@@ -800,3 +810,123 @@ void MAdministrator::toggle_sidebar_cb()
     _xml->get_widget("scrolledwindow1")->hide();
 }
  */
+
+
+void MAdministrator::update_edit_menu()
+{
+  Gtk::Widget *widget= _win->get_focus();
+  bool cut= false;
+  bool copy= false;
+  bool paste= false;
+  
+  if (!widget) return;
+  
+  if (GTK_IS_LABEL(widget->gobj()))
+  {
+    Gtk::Label *label= (Gtk::Label*)widget;
+    int s, e;
+
+    if (label->get_selection_bounds(s,e) > 0)
+    {
+      copy= true;
+    }
+  }
+  else if (GTK_IS_EDITABLE(widget->gobj()))
+  {
+    Gtk::Editable *editable= (Gtk::Editable*)widget;
+    int s, e;
+    
+    if (editable->get_selection_bounds(s,e) > 0)
+    {
+      cut= true;
+      copy= true;
+    }
+    if (Gtk::Clipboard::get()->wait_is_text_available() && editable->get_editable())
+      paste= true;
+  }
+  else if (GTK_IS_TEXT_VIEW(widget->gobj()))
+  {
+    Gtk::TextView *tview= (Gtk::TextView*)widget;
+    Gtk::TextIter s, e;
+    
+    if (tview->get_buffer()->get_selection_bounds(s, e))
+    {
+      cut= true;
+      copy= true;
+    }
+    if (Gtk::Clipboard::get()->wait_is_text_available() && tview->get_editable())
+      paste= true;
+  }
+  _xml->get_menu_item("cut1")->set_sensitive(cut);
+  _xml->get_menu_item("copy1")->set_sensitive(copy);
+  _xml->get_menu_item("paste1")->set_sensitive(paste);
+}
+
+
+void MAdministrator::cut_cb()
+{
+  Gtk::Widget *widget= _win->get_focus();
+  if (!widget) return;
+  if (GTK_IS_EDITABLE(widget->gobj()))
+  {
+    Gtk::Editable *editable= (Gtk::Editable*)widget;
+
+    editable->cut_clipboard();
+  }
+  else if (GTK_IS_TEXT_VIEW(widget->gobj()))
+  {
+    Gtk::TextView *tview= (Gtk::TextView*)widget;
+
+    tview->get_buffer()->cut_clipboard(Gtk::Clipboard::get());
+  }
+}
+
+
+void MAdministrator::copy_cb()
+{
+  Gtk::Widget *widget= _win->get_focus();
+  if (!widget) return;
+  if (GTK_IS_LABEL(widget->gobj()))
+  {
+    Gtk::Label *label= (Gtk::Label*)widget;
+    Glib::ustring text= label->get_label();
+    int s, e;
+    
+    if (label->get_selection_bounds(s,e))
+    {
+      Gtk::Clipboard::get()->set_text(text.substr(s, e));
+    }
+
+  }
+  else if (GTK_IS_EDITABLE(widget->gobj()))
+  {
+    Gtk::Editable *editable= (Gtk::Editable*)widget;
+
+    editable->copy_clipboard();
+  }
+  else if (GTK_IS_TEXT_VIEW(widget->gobj()))
+  {
+    Gtk::TextView *tview= (Gtk::TextView*)widget;
+
+    tview->get_buffer()->copy_clipboard(Gtk::Clipboard::get());
+  }
+}
+
+
+void MAdministrator::paste_cb()
+{
+  Gtk::Widget *widget= _win->get_focus();
+  if (!widget) return;
+  if (GTK_IS_EDITABLE(widget->gobj()))
+  {
+    Gtk::Editable *editable= (Gtk::Editable*)widget;
+
+    editable->paste_clipboard();
+  }
+  else if (GTK_IS_TEXT_VIEW(widget->gobj()))
+  {
+    Gtk::TextView *tview= (Gtk::TextView*)widget;
+
+    tview->get_buffer()->paste_clipboard(Gtk::Clipboard::get());
+  }
+}
