@@ -50,7 +50,7 @@
 
 #include "MAdministrator.h"
 
-#include "myx_aux_functions.h"
+#include "myx_util_functions.h"
 #include "myx_public_interface.h"
 #include "myx_admin_public_interface.h"
 
@@ -182,7 +182,7 @@ MInstanceInfo::MInstanceInfo(MAdministrator *main_win)
 {
   pthread_mutex_init(&_mysql_mx, NULL);
 
-  _dispatcher.signal_work_start_stop().connect(SigC::slot(*main_win, &MAdministrator::set_busy_progress));
+  _dispatcher.signal_work_start_stop().connect(sigc::mem_fun(*main_win, &MAdministrator::set_busy_progress));
 }
 
 
@@ -236,6 +236,7 @@ void MInstanceInfo::set_connection(MYSQL *mysql,
                      "Please note that older versions are not supported and "
                      "may not work properly."), 
                    major_version, myx_get_mysql_minor_version(mysql)),
+              false,
               Gtk::MESSAGE_WARNING,
               Gtk::BUTTONS_OK,
               true);
@@ -290,27 +291,26 @@ MYSQL *MInstanceInfo::clone_mysql()
 }
 
 
-bool MInstanceInfo::check_connection(MYSQL *mysql)
+bool MInstanceInfo::check_connection()
 {
   bool gone= false;
 
   if (_lost_connection_msg_shown)
     return false;
 
-  switch (myx_mysql_errno(mysql ?: _mysql))
+  switch (myx_mysql_errno(_mysql))
   {
   case CR_SERVER_GONE_ERROR:
   case CR_CONNECTION_ERROR:
   case CR_SERVER_LOST:
   case CR_CONN_HOST_ERROR:
-    g_message("Lost connectionto server");
     gone= true;
     break;
   }
 
   if (!gone)
   {
-    if (0&&mysql_ping(mysql ?: _mysql) != 0)
+    if (0&&mysql_ping(_mysql) != 0)
     {
       gone= true;
     }
@@ -318,11 +318,37 @@ bool MInstanceInfo::check_connection(MYSQL *mysql)
 
   if (gone)
   {
-    if (!mysql)
+    Gtk::MessageDialog dlg(_("<b>The Connection to the MySQL server was lost.</b>\nA server shut down or timeout are possible causes. You may try reconnecting."), true,
+                           Gtk::MESSAGE_ERROR, Gtk::BUTTONS_NONE, true);
+
+//    _lost_connection_msg_shown= true;
+    
+    dlg.add_button(_("Quit"), 1);
+    dlg.add_button(_("_Ignore"), 0);
+    dlg.add_button(_("_Reconnect"), 2);
+    
+    switch (dlg.run())
     {
-      disconnect();
-      _lost_connection_msg_shown= true;
-      myg_show_error(_("The connection to the MySQL server was lost.\nPlease reconnect."));
+    case 0:
+      break;
+      
+    case 1:
+      _main_win->quit();
+      break;
+      
+    case 2:
+      MYX_USER_CONNECTION conn;
+      _user_conn.fill(&conn);
+      
+      if (!_mysql)
+        _mysql= myx_mysql_init();
+      
+      if (myx_connect_to_instance(&conn, _mysql) < 0)
+      {
+        myg_show_mysql_error(_("Could not reconnect to MySQL server."), _mysql);
+        return false;
+      }
+      return true;
     }
     return false;
   }
@@ -647,7 +673,7 @@ void MInstanceInfo::start_stop_service(bool stop,
   FILE *output= myx_popen((char**)ping_args, &pid);
   int p= 0;
 
-  _main_win->push_stop_button_handler(SigC::bind<pid_t,int*>(SigC::slot(*this,&MInstanceInfo::cancel_start_stop), pid, &p));
+  _main_win->push_stop_button_handler(sigc::bind<pid_t,int*>(sigc::mem_fun(*this,&MInstanceInfo::cancel_start_stop), pid, &p));
 
   if (!output)
   {
@@ -659,14 +685,14 @@ void MInstanceInfo::start_stop_service(bool stop,
   }
   else
   {
-    input_handler= Glib::signal_io().connect(SigC::bind<FILE*,Glib::RefPtr<Gtk::TextBuffer> >
-                                             (SigC::slot(*this, &MInstanceInfo::process_service_output),output,text),
+    input_handler= Glib::signal_io().connect(sigc::bind<FILE*,Glib::RefPtr<Gtk::TextBuffer> >
+                                             (sigc::mem_fun(*this, &MInstanceInfo::process_service_output),output,text),
                                              fileno(output),
                                              Glib::IO_IN|Glib::IO_HUP|Glib::IO_ERR);
   }
 
   // check whether the child process is still there
-  Glib::signal_timeout().connect(SigC::bind<pid_t>(SigC::slot(check_child),pid),500);
+  Glib::signal_timeout().connect(sigc::bind<pid_t>(sigc::ptr_fun(check_child),pid),500);
 
   Gtk::Main::instance()->run();
 
@@ -716,16 +742,16 @@ void *MInstanceInfo::fetch_server_info(MYSQL *mysql, void *data)
   MInstanceInfo *info= static_cast<MInstanceInfo*>(data);
   
   value= myx_get_server_variable(mysql, "datadir");
+  info->_var_datadir= value;
   if (value)
   {
-    info->_var_datadir= value;
     g_free(value);
   }
 
   value= myx_get_server_variable(mysql, "log_error");
+  info->_var_errorlog= value;
   if (value)
   {
-    info->_var_errorlog= value;
     g_free(value);
   }
 
@@ -737,11 +763,11 @@ MInstanceInfo::ServerState MInstanceInfo::get_server_state()
 {
   //XXX replace with instance manager thing
 
-  if (is_connected())
-    return SRunning;
-
   if (!is_local())
   {
+    if (is_connected())
+      return SRunning;
+
     return SUnknown;
   }
   
@@ -930,7 +956,7 @@ std::string MInstanceInfo::get_server_path(ServerPath type)
       }
 
       // try in my.cnf
-      if (tmp.empty())
+      if (tmp.empty() && mycnf)
       {
         path= myx_get_cnf_value(mycnf, "mysqld", "err-log", &err);
         if (path)
@@ -967,13 +993,15 @@ std::string MInstanceInfo::get_server_path(ServerPath type)
     if (_local_info && !_local_info->general_log_path.empty())
       return _local_info->general_log_path;
     {
-      path= myx_get_cnf_value(mycnf, "mysqld", "log", &err);
-      if (path)
+      if (mycnf)
       {
-        tmp= path;
-        free(path);
+        path= myx_get_cnf_value(mycnf, "mysqld", "log", &err);
+        if (path)
+        {
+          tmp= path;
+          free(path);
+        }
       }
-      
       // try /var/lib/mysql/hostname.log
       if (tmp.empty())
       {
