@@ -84,8 +84,8 @@ class MInstanceInfo : public Glib::ObjectBase {
  private:
     MDispatcher _dispatcher;
 
-    SigC::Signal0<void> _disconnect_signal;
-    SigC::Signal0<void> _connect_signal;
+    sigc::signal<void> _disconnect_signal;
+    sigc::signal<void> _connect_signal;
 
     int _mysql_tid;
     
@@ -115,7 +115,7 @@ class MInstanceInfo : public Glib::ObjectBase {
     ~MInstanceInfo();
     void disconnect();
     
-    bool check_connection(MYSQL *mysql= 0);
+    bool check_connection();
     
     void set_connection(MYSQL *mysql, const MYX::UserConnection &user_conn);
     void refresh_server_info();
@@ -175,8 +175,8 @@ class MInstanceInfo : public Glib::ObjectBase {
     
     ServerState get_server_state();
 
-    SigC::Signal0<void> signal_disconnect() { return _disconnect_signal; };
-    SigC::Signal0<void> signal_connect() { return _connect_signal; };
+    sigc::signal<void> signal_disconnect() { return _disconnect_signal; };
+    sigc::signal<void> signal_connect() { return _connect_signal; };
 };
 
 
