@@ -7,6 +7,7 @@
 
 @interface MACatalogs : MAPanel
 {
+  IBOutlet NSTabView *tabView;
   IBOutlet NSScrollView *tableScrollView;
   IBOutlet NSTableView *tablesTable;
   IBOutlet NSButton *tableDetailToggleButton;
@@ -25,13 +26,29 @@
   IBOutlet NSSearchField *schemataSearch;
   IBOutlet NSOutlineView *schemataOutline;
   
+  IBOutlet NSTableView *viewTable;
+  
+  IBOutlet NSTableView *spTable;
+  IBOutlet NSTableView *funcTable;
+  
   NSImage *_tableIcon;
   NSImage *_indexIcon;
   NSImage *_columnIcon;
+  NSImage *_viewIcon;
+  NSImage *_procIcon;
+  NSImage *_funcIcon;
   
   MFilteredSchemaDataSource *_schemaDS;
   
-  MYX_SCHEMA_TABLE_STATUS *_tableStatus;
+  MYX_SCHEMA_ENTITY_STATUS *_objectStatus;
+  int *_tableIndices;
+  int *_viewIndices;
+  int *_procIndices;
+  int *_funcIndices;
+  unsigned int _tableCount;
+  unsigned int _viewCount;
+  unsigned int _funcCount;
+  unsigned int _procCount;
 }
 
 - (IBAction)toggleTableDetails:(id)sender;
@@ -41,9 +58,18 @@
 - (IBAction)createTable:(id)sender;
 - (IBAction)editTable:(id)sender;
 - (IBAction)dropTable:(id)sender;
-- (IBAction)refreshTables:(id)sender;
+- (IBAction)refreshSchemas:(id)sender;
+- (IBAction)refreshContents:(id)sender;
 - (IBAction)searchCatalogs:(id)sender;
 
+- (IBAction)createView:(id)sender;
+- (IBAction)editView:(id)sender;
+- (IBAction)dropView:(id)sender;
+
+- (IBAction)createProcedure:(id)sender;
+- (IBAction)editProcedure:(id)sender;
+- (IBAction)dropProcedure:(id)sender;
+
 - (IBAction)addSchema:(id)sender;
 - (IBAction)delSchema:(id)sender;
 
