@@ -17,6 +17,7 @@
 
 @class MAPanel;
 @class MSchemaDataSource;
+@class MConnectionInfo;
 
 
 extern NSString *MASchemaDataSourceChanged;
@@ -38,7 +39,7 @@ extern NSString *MASchemaDataSourceChanged;
   NSImage *_keyIcon;
 
   NSLock *_mysqlLock; // remove
-  MYX_USER_CONNECTION *_info;
+  MConnectionInfo *_info;
   MYSQL *_mysql; // remove
   
   MMySQLDispatcher *_dispatcher;
@@ -54,7 +55,7 @@ extern NSString *MASchemaDataSourceChanged;
 
 // gains ownership of mysql and info
 - (id)initWithConnection: (MYSQL*)mysql
-                    info: (MYX_USER_CONNECTION*)info;
+                    info: (MConnectionInfo*)info;
 
 
 - (void)toolbarItemClicked:(id)sender;
@@ -80,7 +81,7 @@ extern NSString *MASchemaDataSourceChanged;
 - (void)unlockMySQL;
 - (MYSQL*)mysql;
 - (MYSQL*)mysqlLock;
-- (MYX_USER_CONNECTION*)serverInfo;
+- (MConnectionInfo*)serverInfo;
 
 - (MMySQLDispatcher*)dispatcher;
 
