@@ -10,18 +10,29 @@
 
 #include "myx_admin_public_interface.h"
 
+
+typedef enum {
+  PGlobal,
+  PSchema,
+  PTable,
+  PColumn
+} PrivilegeType;
+
 @interface PrivilegeDataSource : NSObject
 {
   NSArray *_allPrivileges;
   NSMutableArray *_assignedPrivileges;
   MYX_USER_OBJECT_PRIVILEGES *_privileges;
   
+  PrivilegeType _type;
+  
   NSTableView *_sourceTable;
 }
 
 - (void)dealloc;
 - (id)initWithPrivileges: (NSArray*)privileges
-             sourceTable: (NSTableView*)table;
+             sourceTable: (NSTableView*)table
+                    type: (PrivilegeType)type;
 
 - (void)setPrivileges: (MYX_USER_OBJECT_PRIVILEGES*)objPrivileges;
 - (void)revoke: (NSString*)priv;
