@@ -24,7 +24,7 @@
 #include <libxml/tree.h>
 
 #include <myx_library.h>
-#include <myx_xml_aux_functions.h>
+#include <myx_xml_util_functions.h>
 
 typedef enum _kind { 
   APPLICATION_SPECIFIC_FILE=  1,
@@ -149,11 +149,11 @@ MYX_TRANS_TEXT * find_translation(unsigned int textgroups_num,
                                   const char *id_text)
 {
   MYX_TRANS_TEXTGROUP *tg;
-  MYX_TRANS_TEXTGROUP tmp_tg= {(char *)id_textgroup,0,NULL};
-  MYX_TRANS_TEXT tmp_text= {(char *)id_text, NULL};
+  MYX_TRANS_TEXTGROUP tmp_tg= {(unsigned char *)id_textgroup,0,NULL};
+  MYX_TRANS_TEXT tmp_text= {(unsigned char *)id_text, NULL};
 
   if (*current_textgroup != NULL &&
-      !xmlStrcmp((*current_textgroup)->id, id_textgroup) )
+      !xmlStrcmp((*current_textgroup)->id, (xmlChar*)id_textgroup) )
   {
     tg= *current_textgroup;
   }
@@ -382,7 +382,7 @@ static int parse_translation_file(MYX_TRANS *trans, const char *filename,
     return -1;
   }
 
-  groups_num= get_child_count(root, "textgroup");
+  groups_num= get_child_count(root, (xmlChar*)"textgroup");
   if (groups_num > 0)
     group= calloc(groups_num, sizeof(MYX_TRANS_TEXTGROUP));
 
@@ -402,7 +402,7 @@ static int parse_translation_file(MYX_TRANS *trans, const char *filename,
 
   for (cur= root->children; cur; cur= cur->next)
   {
-    if (!xmlStrcmp(cur->name, "textgroup") )
+    if (!xmlStrcmp(cur->name, (xmlChar*)"textgroup") )
     {
       read_in_textgroup(cur, group, lang);
       group++;
@@ -424,16 +424,16 @@ static void read_in_textgroup(const xmlNodePtr textgroup_node,
   MYX_TRANS_TEXT * text;
   xmlNodePtr cur;
 
-  group->id= xmlGetProp(textgroup_node, "id");
+  group->id= xmlGetProp(textgroup_node, (xmlChar*)"id");
 
-  group->texts_num= get_child_count(textgroup_node, "text");
+  group->texts_num= get_child_count(textgroup_node, (xmlChar*)"text");
   if (group->texts_num > 0)
     group->texts= calloc(group->texts_num, sizeof(MYX_TRANS_TEXT) );
 
   text= group->texts;
   for (cur= textgroup_node->children; cur; cur= cur->next)
   {
-    if ( !xmlStrcmp(cur->name, "text") &&  cur->type == XML_ELEMENT_NODE)
+    if ( !xmlStrcmp(cur->name, (xmlChar*)"text") &&  cur->type == XML_ELEMENT_NODE)
     {
       read_in_text(cur, text, lang);
       text++;
@@ -454,16 +454,16 @@ static void read_in_text(const xmlNodePtr text_node,
   xmlNodePtr cur;
   xmlChar *tmp;
 
-  text->id= xmlGetProp(text_node, "id");
+  text->id= xmlGetProp(text_node, (xmlChar*)"id");
 
   for (cur= text_node->children; cur; cur = cur->next)
   {
-    if (! xmlStrcmp(cur->name, "trans") )
+    if (! xmlStrcmp(cur->name, (xmlChar*)"trans") )
     {
-      tmp= xmlGetProp(cur, "lang");
-      if (! xmlStrcmp(tmp, lang) )
+      tmp= xmlGetProp(cur, (xmlChar*)"lang");
+      if (! xmlStrcmp(tmp, (xmlChar*)lang) )
       {
-        text->trans = xmlNodeListGetString(doc,cur->children,1);
+        text->trans = (char*)xmlNodeListGetString(doc,cur->children,1);
         xmlFree(tmp);
         break;
       }
