@@ -25,7 +25,6 @@
 
 #include <png.h>
 
-#include "myx_gc.h"
 #include "myx_gc_datatypes.h"
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -39,15 +38,15 @@ typedef enum tagColorType
   COLOR_TYPE_RGB_ALPHA =   PNG_COLOR_TYPE_RGB_ALPHA
 } TColorType;
 
-typedef struct tagTImage
+typedef struct tagImage
 {
-  unsigned int Width;     // The width of the image in pixels.
-  unsigned int Height;    // The height of the image in pixels.
-  unsigned char* Data;    // The image data.
-  TColorType ColorType;    // The color format of the image (RGB, gray, palette etc.)
-                           // Note: Palette images are not supported.
-  unsigned int Channels;  // Bytes per pixel.
-  GLenum Format;           // OpenGL color format specifier. Set by the image user.
+  int width;              // The width of the image in pixels.
+  int height;             // The height of the image in pixels.
+  unsigned char* data;   // The image data.
+  TColorType colorType;   // The color format of the image (RGB, gray, palette etc.)
+                          // Note: Palette images are not supported.
+  unsigned int channels; // Bytes per pixel.
+  GLenum format;          // OpenGL color format specifier. Set by the image user.
 } TImage;
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -64,35 +63,45 @@ GENERIC_CANVAS_API string utf16ToANSI(const wstring& Source);
 /** Converts the given string, which is supposed to be an UTF-8 encoded text into an ANSI string using the current system locale. */
 GENERIC_CANVAS_API string utf8ToANSI(const string& Source);
 
+/** Saves the given image as a PNG image. */
+GENERIC_CANVAS_API bool savePNG(const wstring& filename, TImage *image, bool flipped, const char* titel, const char* software);
+
 /** Loads the given PNG image from disk. */
-GENERIC_CANVAS_API TImage* loadPNG(const string& Filename);
+GENERIC_CANVAS_API TImage* loadPNG(const wstring& filename);
 
 /** Releases the given image. */
-GENERIC_CANVAS_API void freeImage(TImage* Image);
+GENERIC_CANVAS_API void freeImage(TImage* image);
 
 /** Returns the current working folder. */
 GENERIC_CANVAS_API string getCurrentDir(void);
 
 /** Sets the current working folder. */
-GENERIC_CANVAS_API void setCurrentDir(const string& Folder);
+GENERIC_CANVAS_API void setCurrentDir(const string& folder);
 
 /** Extracts the drive and path from the given file name. */
-GENERIC_CANVAS_API string extractFilePath(const string& Filename);
+GENERIC_CANVAS_API string extractFilePath(const string& filename);
 
 /** Platform neutral file open function. */
-FILE* openFile(string Filename, const char* OpenMode);
+FILE* openFile(string filename, const char* openMode);
+FILE* openFile(wstring filename, const wchar_t* openMode);
+
+/** Read a text file into memory and perform $variable$ substitutions */
+char *preprocessFile(const string &filename, map<string,string>& variables, size_t &size);
 
 /** Sorts the given bounds so that left <= right and bottom <= top. */
-TBoundingBox sortBounds(const TBoundingBox& Bounds);
+TBoundingBox sortBounds(const TBoundingBox& bounds);
 
 /** Determines whether both bounds overlap. */
-bool boundsIntersect(const TBoundingBox& Bounds1, const TBoundingBox& Bounds2);
+bool boundsIntersect(const TBoundingBox& bounds1, const TBoundingBox& bounds2);
+
+/** Determines whether both the second bounds are within the first bounds. */
+bool boundsContainBounds(const TBoundingBox& bounds1, const TBoundingBox& bounds2);
 
 /** Determines whether bounds are empty. */
-bool boundsAreEmpty(const TBoundingBox& Bounds);
+bool boundsAreEmpty(const TBoundingBox& bounds);
 
 /** Checks if a given point is within the given bounds. */
-bool boundsContainPoint(const TBoundingBox& Bounds, const float X, const float Y);
+bool boundsContainPoint(const TBoundingBox& bounds, const float x, const float y);
 
 /** Converts a container name to an identifier suitable for quick lookup. */
 GENERIC_CANVAS_API TContainerID getContainerID(const string& container);
@@ -104,64 +113,104 @@ GENERIC_CANVAS_API TPropertyID getPropertyID(const string& property);
 int getEntryIndex(string& Path);
 
 /** Converts a (byte) color to a string. */
-string colorToString(GLubyte* Color);
+string colorToString(GLubyte* color);
 
 /** Converts a (float) color to a string. */
-string colorToString(GLfloat* Color);
+string colorToString(GLfloat* color);
 
 // Adds colors to the named color table.
-void registerSystemColors(const TColorMap &ColorMap);
+void registerSystemColors(const CColorMap& colorMap);
 
 // Find a color by name.
-bool colorByName(string Name, GLubyte* Color);
+bool colorByName(string name, GLubyte* color);
 
 /** Converts a string to color with byte members. */
-int stringToColor(string ColorString, GLubyte* Color);
+int stringToColor(string ColorString, GLubyte* color);
 
 /** Converts a string to color with float members. */
-int stringToColor(string ColorString, GLfloat* Color);
+int stringToColor(string ColorString, GLfloat* color);
 
-/** Conversion functions for GC variants and simple values. */
-GENERIC_CANVAS_API string variantToString(const TGCVariant& Variant);
-GENERIC_CANVAS_API int variantToInt(const TGCVariant& Variant);
-GENERIC_CANVAS_API float variantToFloat(const TGCVariant& Variant);
-GENERIC_CANVAS_API bool variantToBool(const TGCVariant& Variant);
-GENERIC_CANVAS_API TGCVariant variant(const string& Value);
-GENERIC_CANVAS_API TGCVariant variant(const char* Value);
-GENERIC_CANVAS_API TGCVariant variant(const int Value);
-GENERIC_CANVAS_API TGCVariant variant(const float Value);
-GENERIC_CANVAS_API TGCVariant variant(const bool Value);
-
-/** Matrix code */
-static void matrixMultiply(TMatrix product, const TMatrix a, const TMatrix b);
+/** matrix code */
+void matrixMultiply(TMatrix product, const TMatrix a, const TMatrix b);
 void matrixRotate(TMatrix mat, GLfloat angle, GLfloat x, GLfloat y, GLfloat z);
 void matrixScale(TMatrix mat, GLfloat x, GLfloat y, GLfloat z);
 void matrixTranslate(TMatrix mat, GLfloat x, GLfloat y, GLfloat z);
-TVertex matrixTransform(TMatrix M, TVertex V);
+TVertex matrixTransform(TMatrix m, const TVertex& v);
+bool matrixInvert(const TMatrix& input, TMatrix& output);
 
 //----------------------------------------------------------------------------------------------------------------------
 
 /**
  * The bounding box computer is a neat little helper class to construct a final bound box out of an arbitrary number of
- * other boxes as well as (lists of) points.
+ * other boxes as well as (lists of) points and vertices.
  */
 class CBoundingBoxComputer
 {
 private:
   TBoundingBox FCurrentBox;
+  bool FEmpty;
 protected:
-  void includeVertex(const TVertex& Vertex);
-  TVertex transform(TMatrix Matrix, const float& X, const float& Y);
+  void includeVertex(const TVertex& vertex);
 public:
   CBoundingBoxComputer(void);
-  CBoundingBoxComputer(const TBoundingBox& InitialBox);
+  CBoundingBoxComputer(const TBoundingBox& initialBox);
 
-  TBoundingBox boundingBox();
-  void include(TMatrix Matrix, TBoundingBox* NewBox);
-  void include(TMatrix Matrix, const TVertex& Vertex);
-  void include(TMatrix Matrix, const float& X, const float& Y);
+  TBoundingBox boundingBox(void);
+  void include(TMatrix matrix, const TBoundingBox& newBox);
+  void include(const TVertex& offset, const TBoundingBox& newBox);
+  void include(TMatrix matrix, const TVertex& vertex);
+  void include(TMatrix matrix, const float& x, const float& y);
+  void reset(void) { FEmpty = true; };
 };
  
 //----------------------------------------------------------------------------------------------------------------------
 
+/**
+ * Simple tokenizer class that works similar as Java's StringTokenizer.
+ */
+class StringTokenizer
+{
+  string FInput, FDelimiters;
+  string::size_type FPosition;
+  char FLastDelimiter;
+public:
+  StringTokenizer(string text, string delimiters): FInput(text), FDelimiters(delimiters)
+  {
+    FPosition = FInput.find_first_not_of(FDelimiters);
+    FLastDelimiter = '\0';
+  };
+  
+  bool hasMoreTokens()
+  {
+    return FPosition != string::npos;
+  };
+  
+  char lastDelimiter(void) { return FLastDelimiter; };
+
+  string nextToken()
+  {
+    string::size_type endPosition = FInput.find_first_of(FDelimiters, FPosition);
+    if (FPosition == string::npos)
+      FLastDelimiter = '\0';
+    else
+      FLastDelimiter = FInput[endPosition];
+    string token = FInput.substr(FPosition, endPosition - FPosition);
+    FPosition = FInput.find_first_not_of(FDelimiters, endPosition);
+
+    return token;
+  };
+
+  float nextTokenAsFloat()
+  {
+    return (float) atof(nextToken().c_str());
+  };
+
+  int nextTokenAsInt()
+  {
+    return atoi(nextToken().c_str());
+  };
+};
+
+//----------------------------------------------------------------------------------------------------------------------
+
 #endif // __GC_GL_UTILITIES_H__
